-- Additional file for the DKOne WOW addon
-- Author: Zorben on Proudmoore-US

-- Addon bails if character is not a DK
if select(2,UnitClass("player"))  ~= "DEATHKNIGHT" then 
	print("|cFFFF0000DKOne is an addon intended for DK's only. You should probably disable it on this toon to save memory if it doesn't do so automatically.|r")
	DisableAddOn("DKOne")
	return
end
local addonName, addonTable = ... --Pulls back the Addon-Local Variables and stores them locally
-- variables used in various ability functions
local cd, start, duration, buffdur, stacks, _ = 0, 0, 0, 0, 0, 0
-- Blizzard API Functions copied as local functions
local setmetatable = setmetatable
local GetItemInfo = GetItemInfo
local GetSpellInfo = GetSpellInfo
local GetItemCooldown = GetItemCooldown
local GetSpellCooldown = GetSpellCooldown
local UnitAura = UnitAura

local Ability, abilityList = {}, {}
Ability.__index = Ability

-- Creation Function to add abilities to the ability list
function Ability.Add(key, SpellId, Cost, Cooldown, Lasts, IsItem, Useable, Start, MyCD)
    local ability = {}
    setmetatable(ability, Ability)
    ability.SpellId = SpellId
	ability.Cost = Cost or {0, 0, 0, 0, 0} --Resource cost for ability in order {Runes{B,U,F,D}, Runic Power}
											-- Can be negative if it generates resources, no current abilities require Death runes
											-- Just planning for MoP
	ability.CooldownLength = Cooldown or 0	-- For future use or items maybe, also for icd on trinkets if needed
	ability.Lasts = Lasts or 0	-- How long effect lasts, may be affected by talents etc
	ability.IsItem = IsItem or false	-- Keep track if ability is an item instead of a spell for checking info
	if ability.IsItem then
		ability.Name = GetItemInfo(SpellId)
		_,_,_,_,_,_,_,_,_,ability.Icon = GetItemInfo(SpellId) -- 10th variable
		ability.MaxRange = 0
	else
		ability.Name = GetSpellInfo(SpellId)
		_,_,ability.Icon = GetSpellInfo(SpellId) -- 3rd variable
		_,_,_,_,_,_,_,_,ability.MaxRange = GetSpellInfo(SpellId) -- 9th variable
	end
	ability.Useable = Useable or false	-- Specifically for putting actions on CD for the purposes of calculating next ability
										-- Also tags abilities the player cant use because they are not learned, specced, or talented
	ability.Start = Start or 0	-- For tracking internal cooldowns, this can be set by user and in conjuction with cooldown
								-- used to estimate when a trinket or spell will proc again.
								-- Currently only the Heroic Creche of the Final Dragon is set up to track
	ability.MyCD = MyCD or 0	-- Just a user set number, that can be used for any purpose, Im using it to set and 
								-- track artificial cooldowns created by synching up trinkets and procs for example
	ability.actionButton = nil	-- Which action button has this ability on it
	abilityList[key] = ability	-- Just adding our new ability to the ability list
end


-- This checks that a spell or item will be off cooldown at the time given by the argument
function Ability:OffCD(Time)
	if not self.Useable then -- If we can't use spell then it is not off cooldown
		return false
	end
	-- If its an item we have to use a different API call than if its a spell
	if self.IsItem then 
		start, duration = GetItemCooldown(self.SpellId)
		cd = start + duration - Time
	else
		start, duration = GetSpellCooldown(self.SpellId)
		cd = start + duration - Time
	end
	-- durations less than or equal to 1.5 secs indicate a cooldown triggered by the GCD
	if cd <= 0 or duration <= 1.5 then 
		return true 
	end
	return false
end

-- This checks what the cooldown for a spell or item is at the time given 
function Ability:GetCD(Time)
	if not self.Useable then -- If we can't use spell then we return a cooldown equal to the maximum
		return 999, 0 ,0
	end
	-- If its an item we have to use a different API call than if its a spell
	if self.IsItem then 
		start, duration = GetItemCooldown(self.SpellId)
		cd = start + duration - Time
	else
		start, duration = GetSpellCooldown(self.SpellId)
		cd = start + duration - Time
	end
	-- I filter out cooldowns triggered by the GCD and by those abilities
	-- requiring a rune that is on cooldown
	if duration <= 10 then
		return 0, start, duration
	elseif cd <= 0 then 
		return 0, 0, 0 
	end
	return cd, start, duration
end

-- Checks if the given ability is present as a buff on the player
-- returns true if there is a buff, false if not
-- Stupid Will of Necropolis cant be checked by name, so I added
-- a check by SpellId option, putting true for byId searches by id instead of name
local expires, spellID
function Ability:GetBuff(byId)
	if byId then
		buffdur = nil
		for i = 1,40 do
			_, _, _, _, _, _, expires, _,_, _, spellID = UnitAura("player", i, "HELPFUL")
			if spellID == self.SpellId then
				buffdur = expires
			end
		end
	else
		buffdur = UnitAura("Player", self.Name, nil, "HELPFUL")
	end
	if buffdur == nil then
		return false 
	else
		return true
	end
end

-- Checks if the given ability is present as a buff on the player
-- returns 0 if there is not a buff, or spell isnt a buff, otherwise the time left on buff
function Ability:GetBuffDur(Time)
	_,_,_,_,_,_,buffdur = UnitAura("Player", self.Name, nil, "HELPFUL") -- 7th variable
	if buffdur == nil or buffdur <= 0 then
		return 0 
	else
		return buffdur - Time
	end
end

-- Checks how many stacks of a given buff are present on the player
-- returns 0 if there are no stacks or the ability is not a stacking buff
-- otherwise the number of stacks
function Ability:GetStacks()
	_,_,_,stacks = UnitAura("Player", self.Name, nil, "HELPFUL") -- 4th variable
	if stacks == nil or stacks <= 0 then
		return 0 
	else
		return stacks
	end
end

-- Death Knight abilities
-- Abilities are checked if they are useable when the ACTIVE_TALENT_GROUP_CHANGED()
-- event is triggered (which is triggered on initalization as well).
-- Ability.Add(key(i.e. name), SpellId, Cost = {Blood, Unholy, Frost, Death, RunicPower}, Cooldown, Lasts, IsItem, Useable, Start, MyCD)
Ability.Add("AntiMagicShell",48707, {0, 0, 0, 0, 0}, 45, 5 , false, true) 
Ability.Add("AntiMagicZone", 51052, {0, 0, 0, 0, 0}, 120, 10, false, false) 
Ability.Add("ArmyOfTheDead", 42650, {1, 1, 1, 0, -30}, 600, 4, false, true) -- Lasts is set to 4 secs for casting time when used as a CD
Ability.Add("Asphyxiate", 108194, {0, 0, 0, 0, 0}, 60, 5, false, true)
Ability.Add("BloodBoil", 48721, {1, 0, 0, 0, -10}, 0, 0, false, true)
Ability.Add("BloodPlague", 55078, {0, 0, 0, 0, 0}, 0, 30 , false, true) -- Lasts is the duration of the dot
Ability.Add("BloodPresence", 48263, {0, 0, 0, 0, 0}, 0, 0, false, true)
Ability.Add("BloodShield", 77535, {0, 0, 0, 0, 0}, 0, 10, false, true)
Ability.Add("BloodStrike", 45902, {1, 0, 0, 0, -10}, 0, 0, false, true)
Ability.Add("BloodTap", 45529, {0, 0, 0, 0, 0}, 0, 0, false, true)
Ability.Add("BloodCharge", 114851, {0, 0, 0, 0, 0}, 25 , 0, false, true)
Ability.Add("BoneShield", 49222, {0, 1, 0, 0, -10}, 60, 300, false, false) 
Ability.Add("ChainsOfIce", 45524, {0, 0, 0, 0, 0}, 0, 10, false, false)
Ability.Add("ControlUndead", 111673, {0, 1, 0, 0, -10}, 0, 0, false, true) 
Ability.Add("Conversion", 119975, {0, 0, 0, 0, 10}, 0, 0, false, false) 
-- removed
--Ability.Add("CrimsonScourge", 81141, {0, 0, 0, 0, -10}, 0, 8, false, true)
Ability.Add("DancingRuneWeapon", 49028, {0, 0, 0, 0, 60}, 90, 12, false, false) 
Ability.Add("DarkCommand", 56222, {0, 0, 0, 0, 0}, 8, 3, false, true) -- The fixate induced by the taunt lasts 3 seconds
Ability.Add("DarkSimulacrum", 77606, {0, 0, 0, 0, 20}, 60, 20, false, true) -- Lasts in this case is how long you have to cast the stolen spell
Ability.Add("DarkSuccor", 101568, {0, 0, 0, 0, 0}, 0, 15, false, false) 
Ability.Add("DarkTransformation", 63560, {0, 1, 0, 0, -10}, 0, 30, false, false) -- Transformation on ghoul lasts 30 secs
Ability.Add("DeathAndDecay", 43265, {0, 1, 0, 0, -10}, 30, 10 , false, true)
Ability.Add("DeathCoil", 47541, {0, 0, 0, 0, 40}, 0, 0, false, true) -- TODO Glyph of Death's Embrace? Not sure if it would ever come up
Ability.Add("DeathGate", 50977, {0, 1, 0, 0, 0}, 60, 0, false, true) -- For completeness, cant use in combat
Ability.Add("DeathGrip", 49576, {0, 0, 0, 0, 0}, 35 , 3, false, true) -- Fixate lasts 3 secs
Ability.Add("DeathPact", 48743,  {0, 0, 0, 0, 0}, 120, 0, false, true)
Ability.Add("DeathsAdvance", 96268, {0, 0, 0, 0, 0}, 30, 6, false, true)
Ability.Add("DeathSiphon", 108196, {0, 0, 0, 1, -10}, 0, 6, false, true)
Ability.Add("DeathStrike", 49998, {0, 1, 1, 0, -20}, 0, 0, false, true)
Ability.Add("DesecratedGround", 108201, {0, 0, 0, 0, 0}, 120, 0, false, false)
Ability.Add("EmpowerRuneWeapon", 47568, {0, 0, 0, 0, -25}, 300, 0, false, true) -- Costs include the RP generated but not the runes
Ability.Add("FesteringStrike", 85948, {1, 0, 1, 0, -20}, 0, 0, false, true)
Ability.Add("FrostFever", 55095, {0, 0, 0, 0, 0}, 0, 30 , false, true) -- Lasts is the duration of the dot
Ability.Add("FrostPresence", 48266, {0, 0, 0, 0, 0}, 0, 0, false, true)
Ability.Add("FrostStrike", 49143, {0, 0, 0, 0, 35}, 0, 0, false, false)
Ability.Add("GorefiendsGrasp", 108199, {0, 0, 0, 0, 0}, 60, 0, false, false)
Ability.Add("HeartStrike", 55050, {1, 0, 0, 0, -10}, 0, 0, false, true)
Ability.Add("HornOfWinter", 57330, {0, 0, 0, 0, -10}, 20, 120 , false, true)
Ability.Add("HowlingBlast", 49184, {0, 0, 1, 0, -10}, 0, 0, false, false)
--removed
--Ability.Add("HungeringCold", 49203, {0, 0, 0, 0, 40 }, 60, 10, false, false)
Ability.Add("IceboundFortitude", 48792, {0, 0, 0, 0, 20}, 180, 12, false, true) 
Ability.Add("IcyTouch", 45477, {0, 0, 1, 0, -10}, 0, 0, false, true)
Ability.Add("KillingMachine", 51124, {0, 0, 0, 0, 0}, 0, 10, false, true)
Ability.Add("Lichborne", 49039, {0, 0, 0, 0, 0}, 120, 10, false, false)
Ability.Add("MightOfUrsoc", 113072, {0, 0, 0, 0, 0}, 180, 20, false, false)
Ability.Add("MindFreeze", 47528, {0, 0, 0, 0, 0}, 10, 4, false, true)
Ability.Add("NecroticStrike", 73975, {0, 0, 0, 1, -10}, 0, 10, false, true) -- Cast Time slow lasts 10 secs
Ability.Add("Obliterate", 49020, {0, 1, 1, 0, -20}, 0, 0, false, true)
Ability.Add("Outbreak", 77575, {0, 0, 0, 0, 0}, 60, 0, false, true)
Ability.Add("PathOfFrost", 3714, {0, 0, 1, 0, 0}, 0, 600, false, true) 
Ability.Add("Pestilence", 50842, {1, 0, 0, 0, -10}, 0, 0, false, true)
Ability.Add("PillarOfFrost", 51271, {0, 0, 1, 0, -10}, 60, 20, false, false)
Ability.Add("PlagueStrike", 45462, {0, 1, 0, 0, -10}, 0, 0, false, true)
Ability.Add("PlagueLeech", 123693, {0, 0, 0, 0, 0}, 25, 0, false, true)
Ability.Add("RaiseAlly", 61999, {0, 0, 0, 0, 30}, 600, 0, false, true)
Ability.Add("RaiseDead", 46584, {0, 0, 0, 0, 0}, 180, 60, false, true)
Ability.Add("RemorselessWinter", 108200, {0, 0, 0, 0, 0}, 60, 0, false, false)
Ability.Add("Rime", 59052, {0, 0, 0, 0, 0}, 0, 10, false, true) -- Freezing Fog
Ability.Add("RuneOfTheFallenCrusader", 53365, {0, 0, 0, 0, 0}, 0, 15, false, true)
Ability.Add("RuneStrike", 56815, {0, 0, 0, 0, 30}, 30, 0, false, true)
Ability.Add("RuneTap", 48982, {1, 0, 0, 0, 0}, 30, 0, false, false)
Ability.Add("RunicCorruption", 51462, {0, 0, 0, 0, 0}, 0, 3, false, false)
Ability.Add("ScarletFever", 81132, {0, 0, 0, 0, 0}, 0, 21, false, false) 
Ability.Add("ScentOfBlood", 50421, {0, 0, 0, 0, 0}, 0, 20, false, false) 
Ability.Add("ScourgeStrike", 55090, {0, 1, 0, 0, -10}, 0, 0, false, false)
Ability.Add("ShadowInfusion", 91342, {0, 0, 0, 0, 0}, 0, 30, false, true)
Ability.Add("ShroudOfPurgatory", 116888, {0, 0, 0, 0, 0}, 0, 30, false, false)
Ability.Add("SoulReaperB", 114866, {1, 0, 0, 0, -10}, 6, 5, false, false)
Ability.Add("SoulReaperF", 130735, {0, 1, 0, 0, -10}, 6, 5, false, false)
Ability.Add("SoulReaperU", 130736, {0, 0, 1, 0, -10}, 6, 5, false, false)
Ability.Add("Strangulate", 47476, {1, 0, 0, 0, -10}, 120 , 0, false, true) 
Ability.Add("SuddenDoom", 81340, {0, 0, 0, 0, 0}, 0, 10, false, true)
Ability.Add("SummonGargoyle", 49206, {0, 0, 0, 0, 60}, 180, 30, false, false)
Ability.Add("UnholyFrenzy", 49016, {0, 0, 0, 0, 0}, 180, 30, false, false)
Ability.Add("UnholyPresence", 48265, {0, 0, 0, 0, 0}, 0, 0, false, true)
Ability.Add("UnholyBlight", 115989, {0, 0, 0, 0, 0}, 90, 10, false, true)
Ability.Add("VampiricBlood", 55233, {0, 0, 0, 0, 0}, 180, 10, false, false)
Ability.Add("WildMushroomPlague", 113516, {0, 0, 0, 0, 0}, 180, 30, false, false)
Ability.Add("WillOfTheNecropolis", 96171, {0, 0, 0, 0, 0}, 15, 8, false, false)

-- DK Pet Abilities The energy cost is the 5th entry in the cost variable
Ability.Add("Claw", 91776, {0, 0, 0, 0, 40}, 0, 0, false, true)
Ability.Add("Gnaw", 91800, {0, 0, 0, 0, 30}, 60, 3, false, true) -- Stun lasts 3 secs
Ability.Add("Huddle", 91838, {0, 0, 0, 0, 10}, 45, 10, false, true)
Ability.Add("Leap", 91809, {0, 0, 0, 0, 10}, 30, 0, false, true)
Ability.Add("MonstrousBlow", 91797, {0, 0, 0, 0, 30}, 60, 4, false, true) -- Ghoul under Dark transformation , gnaw turns into this
Ability.Add("PutridBulwark", 91837, {0, 0, 0, 0, 10}, 45, 10, false, true) -- Empowered Huddle
Ability.Add("ShamblingRush", 91802, {0, 0, 0, 0, 10}, 30, 0, false, true) -- Empowered Leap
Ability.Add("SweepingClaws", 91778, {0, 0, 0, 0, 40}, 0, 0, false, true) -- Empowered Claw

-- Other class abilities
-- Hero
Ability.Add("AncientHysteria", 90355, {0, 0, 0, 0, 0}, 600, 40, false, true)
Ability.Add("Bloodlust", 2825, {0, 0, 0, 0, 0}, 600, 40, false, true)
Ability.Add("Heroism", 32182, {0, 0, 0, 0, 0}, 600, 40, false, true)
Ability.Add("TimeWarp", 80353, {0, 0, 0, 0, 0}, 600, 40, false, true)
-- Attack Power Buff
Ability.Add("BattleShout", 6673, {0, 0, 0, 0, 0}, 60, 120, false, false)
Ability.Add("TrueshotAura", 19506, {0, 0, 0, 0, 0}, 0, 0, false, false)
-- removed
--Ability.Add("StrOfEarth", 8075, {0, 0, 0, 0, 0}, 60, 120, false, false)
-- Attack Speed Slow Debuff
Ability.Add("InfectedWounds", 58180, {0, 0, 0, 0, 0}, 0, 12, false, true)
-- removed
--Ability.Add("JudgementsOfTheJust", 53696, {0, 0, 0, 0, 0}, 0, 20, false, true)
Ability.Add("Thunderclap", 6343, {0, 0, 0, 0, 0}, 0, 30, false, true)
-- Physical Damage Reduction Debuff
Ability.Add("DemoralizingRoar", 99, {0, 0, 0, 0, 0}, 0, 30, false, true)
Ability.Add("Vindication", 26017, {0, 0, 0, 0, 0}, 0, 30, false, true)
Ability.Add("DemoralizingShout", 1160, {0, 0, 0, 0, 0}, 0, 30, false, true)


-- Consumable Buffs
Ability.Add("StrFlaskBuff", 79472, {0, 0, 0, 0, 0}, 0, 0, false, true) -- This is the buff from the Flask of Titanic Strength
Ability.Add("StamFlaskBuff", 92729, {0, 0, 0, 0, 0}, 0, 0, false, true) -- This is the buff from the Flask of Steelskin
Ability.Add("GolembloodBuff", 79634, {0, 0, 0, 0, 0}, 0, 25, false, true)  -- This is the buff from the Golemblood pot, not the actual pot itself
Ability.Add("EarthPotionBuff", 79475, {0, 0, 0, 0, 0}, 0, 25, false, true) -- Buff from Earthen potion is called earthen armor
Ability.Add("StrFoodBuff", 87545, {0, 0, 0, 0, 0}, 0, 0, false, true) -- This is the buff from well fed for str food
Ability.Add("DodgeFoodBuff", 87555, {0, 0, 0, 0, 0}, 0, 0, false, true) -- This is the buff from well fed for dodge food
Ability.Add("ParryFoodBuff", 87554, {0, 0, 0, 0, 0}, 0, 0, false, true) -- This is the buff from well fed for parry food
Ability.Add("MasteryFoodBuff", 87549, {0, 0, 0, 0, 0}, 0, 0, false, true) -- This is the buff from well fed for mastery food

-- Consumable Items
Ability.Add("Seafood", 62290, {0, 0, 0, 0, 0}, 0, 0, true, true) -- This is the food item, generically Seafood Magnifique
Ability.Add("StrFlaskItem", 58088, {0, 0, 0, 0, 0}, 0, 0, true, true) -- This is the Titanic Strength Flask (an item)
Ability.Add("StamFlaskItem", 58085, {0, 0, 0, 0, 0}, 0, 0, true, true) -- This is the Steelskin Flask (an item)
Ability.Add("GolembloodItem", 58146, {0, 0, 0, 0, 0}, 60, 0, true, true) -- This the item for the golemblood pot, need the item in order to check the Cooldown
Ability.Add("EarthPotionItem", 58090, {0, 0, 0, 0, 0}, 60, 0, true, true) -- This the item for the Earthen pot, need the item in order to check the Cooldown

-- Trinket Items
-- MOP Trinkets
Ability.Add("MalvBadge", 84942, {0, 0, 0, 0, 0}, 120, 20, true, true)
-- Cata Trinkets
Ability.Add("RottingSkull", 77116, {0, 0, 0, 0, 0}, 90, 15, true, true)
Ability.Add("FireOfTheDeep", 77117, {0, 0, 0, 0, 0}, 90, 15, true, true)
Ability.Add("BadgeOfVictory", 73496, {0, 0, 0, 0, 0}, 120, 20, true, true)
-- Trinket Buffs
Ability.Add("RageHeart", 91816, {0, 0, 0, 0, 0}, 0, 0, false, true)
Ability.Add("Creche", 109744, {0, 0, 0, 0, 0}, 115, 20, false, true)
Ability.Add("Eye", 109750, {0, 0, 0, 0, 0}, 0, 10, false, true)

-- Racials
Ability.Add("ArcaneTorrent", 50613, {0, 0, 0, 0, -15}, 120, 0, false, false)
Ability.Add("Berserking", 26297, {0, 0, 0, 0, 0}, 180, 10, false, false)
Ability.Add("BloodFury", 20572, {0, 0, 0, 0, 0}, 120, 15, false, false)
Ability.Add("GiftOfNaaru", 59545, {0, 0, 0, 0, 0}, 180, 15, false, false)
Ability.Add("RocketBarrage", 69041, {0, 0, 0, 0, 0}, 120, 0, false, false)
Ability.Add("Stoneform", 20594, {0, 0, 0, 0, 0}, 120, 8, false, false)

-- Non-existent spell
Ability.Add("NoSpell", 0, {0, 0, 0, 0, 0}, 0, 0, false, true)

-- This global function just passes the complete ability list by reference for use in other files
function addonTable.DKOneAbilityList()
	return abilityList
end

-- Add item icons, for some reason Item icons are not loading 
-- before it loads this file, so I go back later and load them again
function addonTable.DKOneLoadItemIcons()
	for k, v in pairs(abilityList) do
		if v.IsItem then
			v.Name = GetItemInfo(v.SpellId)
			_,_,_,_,_,_,_,_,_,v.Icon = GetItemInfo(v.SpellId) -- 10th variable
			v.MaxRange = 0
		end
	end
end
