-- Additional file for the DKOne WOW addon
-- Author: Zorben on Proudmoore-US

-- Addon bails if character is not a DK
if select(2,UnitClass("player"))  ~= "DEATHKNIGHT" then 
	DisableAddOn("DKOne") 
	return
	
end
local addonName, addonTable = ... --Pulls back the Addon-Local Variables and stores them locally
local aL = addonTable.DKOneAbilityList()		-- local reference to the abilityList table
local DKStatus = addonTable.DKOneDKStatus()	-- local reference to the DKStatus table
local DKPlayerStats = addonTable.DKOneDKPlayerStats()	-- local reference to the DKPlayerStats table
local balance	-- local variable for CheckResource()
local temprunecd, tempdrunecd, balance2, tempbalance, stopcheck -- debug variables used in UpdateResource()
-- min, balance is also used in UpdateResource()
local LCost = {0,0,0,0}	-- local copy of the ability cost, used and modified in UpdateResource()
local r1order = {"B1", "U1", "F1"} -- 1st rune of each type, used in UpdateResource()
local r2order = {"B2", "U2", "F2"} -- 2nd rune of each type, used in UpdateResource()
local RPCost = 0		-- Temp variable for runic power cost, used to adjust for frost presence, used in UpdateResource()
local DKQ = {} 			-- Normal queue		set in global function DKOne:UpdateQueue(rotarg)
local DKQAOE = {} 		-- AOE queue		set in global function DKOne:UpdateQueue(rotarg)
local DKQR = {}			-- Ranged Queue		set in global function DKOne:UpdateQueue(rotarg)
local DKQCD = {}		-- Cooldown Queue	set in global function DKOne:UpdateQueue(rotarg)
local runeorder = {"B1", "B2", "U1", "U2", "F1", "F2"} -- Rune Order used in Blizzard API, remember BUF in UpdateStatusTime()
-- max, runeorder, r1order, r2order also used in UpdateStatusTime()
local n -- local variable used in GetNextAction()
local CurrentQ = {}		-- Queue we are currently using, used in GetNextAction()
-- n is also used in GetNextActionCD()
local cd, cd1, cd2, start, start1, start2 		-- local varaibles used in GetNextActionCD()
local duration, duration1, duration2, i1, i2	-- local varaibles used in GetNextActionCD()
local s1, s2, action1, action2 -- local variables used in in the global function DKOne.Rotation()
-- cd1, start1, duration1, cd2, start2, duration2 is also used in RotationCD()
local labelCD1, labelCD2, actionCD1, actionCD2	-- local variables used in RotationCD()
local erwcd, btcd		-- Used in the ActionCD GetCD calls
local pillar, runefc, potion, eyestacks, cretrink, crecd, tempcd, cd, start, duration, erwcd, runes, btcd, runecd
-- FUNCTIONS ------
local min = min	-- local reference to function min used in CheckResource()
local max = max -- local reference to max function used in UpdateResource()
local wipe = wipe	-- local reference to function wipe used in global function DKOne:UpdateQueue(rotarg)
local tinsert = tinsert	-- local reference to function tinsert used in global function DKOne:UpdateQueue(rotarg)
local floor = floor	-- local reference to function floor used in OffCD()
local ceil = ceil -- local reference to ceil function used in UpdateResource()
local string = string	-- local reference to function tinsert used in UpdateQueue()

-- Check if the status variables DKStatus shows enough resources
-- Specifically runes and runic power to cover the costs passed into the function arg
-- Cost Variable is {Blood, Unholy, Frost, Death, RunicPower}
-- return true if resources can cover costs
local function CheckResource(Cost)
	if Cost == nil then -- Cost is Blank
		return true
	end
	if DKStatus.RP < Cost[5] then -- Check Runic Power
		return false -- Runic power isnt covered so bail early
	end
	if  Cost[1] + Cost[2] + Cost[3] + Cost[4] == 0 then -- If the rune costs are 0, we already checked runic power
		return true
	end
	balance = 0 -- For keeping track of how many death runes we need
	-- Now we have to check actual rune costs, this is complicated by death runes
	for i = 1,3 do -- For each type of rune
		balance = balance + min(DKStatus.RuneCD[i] - Cost[i], 0)
	end
	-- here we account for death runes
	balance = balance + DKStatus.DRuneCD[4] - Cost[4]
	if balance < 0 then -- Rune Costs are greater than Rune resources
		return false
	else				-- Rune Costs are covered, and we already checked runic power
		return true
	end
end

-- Update the Resource to account for the given cost
-- Runic Power can be negative for actions that generate runic power
-- However Runes cannot be, so this fuction is not used to update resources after using 
-- Empower Rune Weapon or Blood Tap, the only two abilities that produce runes
-- There is an assumption that the necessary resources exists, there is no robust error checking here
local function UpdateResource(Cost)
	if Cost == nil then -- Cost is Blank
		return -- Just Bail out
	end
	-- First we do the easy one and update runic power
	-- RP generation is increased by 20% in frost presence
	if Cost[5] < 0 and DKStatus.Presence == 2 then
		RPCost = ceil(Cost[5]*1.2)
	else
		RPCost = Cost[5]
	end
	-- We ensure it stays between 0 and MaxRP
	DKStatus.RP = max(0,min(DKPlayerStats.MaxRP, DKStatus.RP - RPCost))
	-- In the case no runes are needed
	if Cost[1] + Cost[2] + Cost[3] + Cost[4] == 0 then
		return -- We are done
	end
	for i = 1,4 do -- For each type of rune
		-- Local copy of cost so we aren't modifying the original cost table
		LCost[i] = Cost[i]
	end
	-- This next code block is tricky, Death runes complicate things
	-- we need to use the runes in the correct order. First we use any non
	-- death runes to cover the costs, then we use Death runes of the correct 
	-- type to cover costs, finally we use death runes from other types to 
	-- cover any remaining costs
	
	-- As an aside to clarify why this complication is necessary, consider a DK casting Army,
	-- this requires 1 rune of each type, now lets say the DK has exactly 3 runes off 
	-- cooldown, the first unholy rune which is also a death rune, the 2nd unholy rune
	-- which isnt a death rune, and the first frost rune which is also a death rune
	-- if you account for the runes in the wrong order you wont end up being able to
	-- cover costs. As far as I can tell this is the order the actual client uses runes up
	for i = 1,3 do
		if LCost[i] == 1 and DKStatus.RuneCD[i] > 0 then -- Check costs, make adjustments
			-- At least 1 rune that isnt a death rune is off CD
			-- by default we use the first rune if its available
			-- r1order and r2order just return the keys for the runes, for example r1order[1] == "B1"
			if DKStatus[r1order[i]] == 0 and DKStatus.NotDeath[2*i - 1] then --Its first rune
				DKStatus[r1order[i]] = DKStatus.RuneDur + DKStatus[r2order[i]] -- putting the one we just used on cooldown
			else -- must be 2nd rune
				DKStatus[r2order[i]] = DKStatus.RuneDur + DKStatus[r1order[i]] -- put it on CD
			end
			DKStatus.RuneCD[i] = DKStatus.RuneCD[i] - 1 -- Subtract 1 from the available runes
			LCost[i] = 0
		end
	end
	-- Now we have checked all the runes that aren't Death runes, we go back through
	-- and use the death runes if they are of the same type as costs
	for i = 1,3 do
		if LCost[i] == 1 and DKStatus.DRuneCD[i] > 0 then -- Check costs of runes, make adjustments
			-- At least 1 rune that is a death rune is off CD
			-- by default we use the first rune if its available
			if DKStatus[r1order[i]] == 0 then --It's first rune
				DKStatus[r1order[i]] = DKStatus.RuneDur + DKStatus[r2order[i]] -- putting the one we just used on cooldown
			else -- must be 2nd rune
				DKStatus[r2order[i]] = DKStatus.RuneDur + DKStatus[r1order[i]] -- put it on CD
			end
			DKStatus.DRuneCD[i] = DKStatus.DRuneCD[i] - 1 -- Subtract 1 from the available death runes of the particular type
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] - 1 -- Subtract 1 from the total available death runes
			LCost[i] = 0
		end
	end
	-- calculate how many death runes we still need
	balance = LCost[1] + LCost[2] + LCost[3] + LCost[4]
	--Finally we use any available death runes
	for i = 1,3 do
		if balance > 0 and DKStatus[r1order[i]] == 0 and not DKStatus.NotDeath[2*i - 1] then -- Rune 1 is a death rune off CD
			DKStatus[r1order[i]] = DKStatus.RuneDur + DKStatus[r2order[i]] -- putting the one we just used on cooldown
			DKStatus.DRuneCD[i] = DKStatus.DRuneCD[i] - 1 -- Subtract 1 from the available death runes of this type
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] - 1 -- Subtract 1 from the total available death runes
			balance = balance - 1
		end
		if balance > 0 and DKStatus[r2order[i]] == 0 and not DKStatus.NotDeath[2*i] then -- Rune 2 is a death rune off CD
			DKStatus[r2order[i]] = DKStatus.RuneDur + DKStatus[r1order[i]] -- putting the one we just used on cooldown
			DKStatus.DRuneCD[i] = DKStatus.DRuneCD[i] - 1 -- Subtract 1 from the available death runes of this type
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] - 1 -- Subtract 1 from the total available death runes
			balance = balance - 1
		end	
	end
end

-- Abilities and Actions
-- Each ability correponds to a single spell, buff, debuff or item.
-- Each action corresponds to the player pushing a button. So every action
-- corresponds to a single ability that may be displayed. Multiple actions
-- may use a given ability, correponding to different circumstances when 
-- you would use that ability. The action fs110 means use Frost Strike if 
-- Runic Power is greater than 110, while the action fs100 means use Frost Strike if 
-- Runic Power is greater than 100.  Both actions fs110 and fs100 link to ability FrostStrike.
-- There is a seprate list of actions called actionsCD for cooldown usage, similar to but not the same as actions

--README IF you add your own actions to the actions list, it must include the following fields
-- key or name should start with letter first although it can contain numbers, they just shouldnt be first
-- It's usually best to at least start with the first letter of the ability it references.
-- abil refers to the abilities listed in the abilities file. The ability must be useable or an item
-- you cant list a buff as an ability for example. It is referenced as aL.abilitykey, 
-- where aL is a local table of all the abilities.
-- order is used to organize the legend on the rotation configuration screens, 0 means used by any spec,
-- 1 is used by blood spec, 2 is used by frost spec, and 3 is used by the unholy spec.
-- OffCD is a function, it has no arguments and returns a simple true or false if the action is ready to be used
-- These functions for each separate action are the heart of the addon really. You can make use of
-- all the status information and other info to set up very intricate conditions as to when
-- certain actions are off CD. 
-- UpdateStatus is a function, it takes no arguments and returns no variables. Generally it 
-- changes the status variables to account for resources used or gained by the action and
-- if the ability called by the action has a cooldown, it sets the ability to not be useable, 
-- so its not being called by the second action if it was called by the first
-- info is just a string describing the action to be taken. It is used in the rotation configuration screens
local actions = {
---------------ANY SPEC---------------------------
	--  Anti Magic Shell
	ams = {
		abil = aL.AntiMagicShell,
		order = 0,
		OffCD = function()
			return aL.AntiMagicShell:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.AntiMagicShell.Useable = false
		end,
		info = "Anti Magic Shell on CD",
	},
	--  Anti Magic Zone (talent)
	amz = {
		abil = aL.AntiMagicZone,
		order = 0,
		OffCD = function()
			return aL.AntiMagicZone:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.AntiMagicZone.Useable = false
		end,
		info = "Anti Magic Zone (talent) on CD",
	},
	-- Army of the dead
	army = {
		abil = aL.ArmyOfTheDead,
		order = 0,
		OffCD = function()
			return CheckResource(aL.ArmyOfTheDead.Cost) and aL.ArmyOfTheDead:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			UpdateResource(aL.ArmyOfTheDead.Cost)
			aL.ArmyOfTheDead.Useable = false
		end,
		info = "Army of the Dead on CD",
	},
	-- Asphyxiate
	asphyx = {
		abil = aL.Asphyxiate,
		order = 0,
		OffCD = function()
			return aL.Asphyxiate:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.Asphyxiate.Useable = false
		end,
		info = "Asphyxiate on CD",
	},
	-- Blood Boil
	bb = {
		abil = aL.BloodBoil,
		order = 0,
		OffCD = function()
			return CheckResource(aL.BloodBoil.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.BloodBoil.Cost)
		end,
		info = "Blood Boil",
	},
	-- Apply Blood Plague using Plague Strike
	bpPS = {
		abil = aL.PlagueStrike,
		order = 0,
		OffCD = function()
			return DKStatus.BP <= 1 and CheckResource(aL.PlagueStrike.Cost) and aL.Outbreak:GetCD(DKStatus.Time) > 3
		end,
		UpdateStatus = function()
			UpdateResource(aL.PlagueStrike.Cost)
			DKStatus.BP  = aL.BloodPlague.Lasts
			if DKPlayerStats.Spec == 1 then
				DKStatus.PDRDebuff = true
			end
		end,
		info = "Apply blood plague using plague strike as long as Outbreak isn't coming off CD very shortly",
	},
	--  Blood Strike
	bs = {
		abil = aL.BloodStrike,
		order = 0,
		OffCD = function()
			return CheckResource(aL.BloodStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.BloodStrike.Cost)
		end,
		info = "Blood Strike",
	},
	-- Blood Tap Usage, 
	-- blood tap behavior is a pain in the ass, I believe this is correct
	bt5 = {
		abil = aL.BloodTap,
		order = 0,
		OffCD = function()
			return DKStatus.BC >= 5 and ((DKStatus.B1 > 0 and DKStatus.B2 > 0) or  (DKStatus.U1 > 0 and DKStatus.U2 > 0) or (DKStatus.F1 > 0 and DKStatus.F2 > 0))
		end,
		UpdateStatus = function()
			-- In MoP, it always finishes a rune, which one it finishes depends on spec
			-- TODO
			--This is close but not right yet I dont think
			-- Blood Spec goes Blood -> Unholy -> Frost
			if DKStatus.Presence == 1 then
				if DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				end
			-- Frost Spec goes Unholy -> Blood -> Frost
			elseif DKStatus.Presence == 2 then
				if DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				elseif DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				end
			-- Unholy Spec goes Blood -> Frost -> Unholy
			elseif DKStatus.Presence == 3 then
				if DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				elseif DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				end
			end
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] + 1
			DKStatus.BC = DKStatus.BC - 5
			--[[-- when none of the blood runes are currently death runes (Unholy and Blood spec)
			if DKStatus.NotDeath[1] and DKStatus.NotDeath[2] and DKStatus.B1 > 0 and DKStatus.B2 > 0 then
				DKStatus.NotDeath[1] = false 
				DKStatus.B1 = 0
			elseif DKStatus.NotDeath[1] and DKStatus.NotDeath[2] and DKStatus.B1 == 0 then
				DKStatus.NotDeath[1] = false 
				DKStatus.RuneCD[1] = DKStatus.RuneCD[1] - 1
			elseif DKStatus.NotDeath[1] and DKStatus.NotDeath[2] and DKStatus.B1 > 0 and DKStatus.B2 == 0 then
				DKStatus.NotDeath[2] = false 
				DKStatus.RuneCD[1] = DKStatus.RuneCD[1] - 1
			-- when both of the blood runes are currently death runes (Frost and Unholy Spec)
			elseif not DKStatus.NotDeath[1] and not DKStatus.NotDeath[2] and DKStatus.B1 > 0 then
				DKStatus.B1 = 0
			elseif not DKStatus.NotDeath[1] and not DKStatus.NotDeath[2] and DKStatus.B1 == 0 and DKStatus.B2 > 0 then
				DKStatus.B2 = 0
			elseif not DKStatus.NotDeath[1] and not DKStatus.NotDeath[2] and DKStatus.B1 == 0 and DKStatus.B2 == 0 then
				DKStatus.DRuneCD[1] = DKStatus.DRuneCD[1] - 1
				DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] - 1
			-- When rune 1 is a death rune and rune 2 is not (Unholy spec only)
			elseif not DKStatus.NotDeath[1] and DKStatus.NotDeath[2] then
				DKStatus.NotDeath[1] = false
				if DKStatus.B1 > 0 then
					DKStatus.B1 = 0
				elseif DKStatus.B2 > 0 then
					DKStatus.B2 = 0
				else
					DKStatus.RuneCD[1] = DKStatus.RuneCD[1] - 1
				end
			-- When rune 2 is a death rune and rune 1 is not (Unholy spec only)
			elseif DKStatus.NotDeath[1] and not DKStatus.NotDeath[2] then
				DKStatus.NotDeath[2] = false
				if DKStatus.B1 > 0 then
					DKStatus.B1 = 0
				elseif DKStatus.B2 > 0 then
					DKStatus.B2 = 0
				else
					DKStatus.RuneCD[1] = DKStatus.RuneCD[1] - 1
				end
			end
			DKStatus.DRuneCD[1] = DKStatus.DRuneCD[1] + 1
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] + 1
			--RuneConCheck("Inside bloodtap action")]]
		end,
		info = "Blood Tap, use when you have 5 charges",
	},
	-- Blood Tap, use when you have 10 charges
	bt10 = {
		abil = aL.BloodTap,
		order = 0,
		OffCD = function()
			return DKStatus.BC >= 10 and ((DKStatus.B1 > 0 and DKStatus.B2 > 0) or  (DKStatus.U1 > 0 and DKStatus.U2 > 0) or (DKStatus.F1 > 0 and DKStatus.F2 > 0))
		end,
		UpdateStatus = function()
			-- In MoP, it always finishes a rune, which one it finishes depends on spec
			-- TODO
			--This is close but not right yet I dont think
			-- Blood Spec goes Blood -> Unholy -> Frost
			if DKStatus.Presence == 1 then
				if DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				end
			-- Frost Spec goes Unholy -> Blood -> Frost
			elseif DKStatus.Presence == 2 then
				if DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				elseif DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				end
			-- Unholy Spec goes Blood -> Frost -> Unholy
			elseif DKStatus.Presence == 3 then
				if DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				elseif DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				end
			end
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] + 1
			DKStatus.BC = DKStatus.BC - 5
		end,
		info = "Blood Tap, use when you have 10 charges",
	},
	-- Conversion when health is below 25% and RP > 40
	conv25 = {
		abil = aL.Conversion,
		order = 0,
		OffCD = function() 
			return DKStatus.RP > 40 and DKStatus.PerHealth < 25
		end,
		UpdateStatus = function()
			DKStatus.PerHealth = DKStatus.PerHealth + 3
			DKStatus.Health = DKStatus.Health + .03*DKStatus.MaxHealth
		end,
		info = "Conversion when health is below 25% and RP > 40",
	},
	-- Deathcoil
	dc = {
		abil = aL.DeathCoil,
		order = 0,
		OffCD = function() 
			return (CheckResource(aL.DeathCoil.Cost) or DKStatus.Doom > 0) and not( DKStatus.Hulk ~= 0 and DKStatus.Hulk < 8 and DKStatus.RP < 100) and not (DKStatus.UF > 0 and aL.SummonGargoyle:OffCD(DKStatus.Time))
		end,
		UpdateStatus = function()
			if DKPlayerStats.Spec == 3 then
				DKStatus.SI = min(5, DKStatus.SI + 1)
			end
			if DKStatus.Doom > 0 then
				DKStatus.Doom = DKStatus.Doom - 1
			else
				UpdateResource(aL.DeathCoil.Cost)
			end
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Deathcoil, unless Dark Transformation is about to end and RP is low",
	},
	--  Self Heal with Death Coil while in Lichborne form if Health less than 80 percent
	dcsh = {
		abil = aL.Lichborne,
		order = 0,
		OffCD = function()
			if aL.Lichborne:GetBuffDur(DKStatus.Time) > 1 then 
				return DKStatus.PerHealth < 80 and CheckResource(aL.DeathCoil.Cost)
			else
				return false
			end
		end,
		UpdateStatus = function()
			DKStatus.PerHealth = DKStatus.PerHealth + 8 -- TODO check actual healing percentage
			DKStatus.Health = DKStatus.Health + 15000
			UpdateResource(aL.DeathCoil.Cost)
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Self Heal with Death Coil while in Lichborne form if Health is less than 80 percent, shows Lichborne icon",
	},
	--  Deaths Advance
	da = {
		abil = aL.DeathsAdvance,
		order = 0,
		OffCD = function()
			return aL.DeathsAdvance:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.DeathsAdvance.Useable = false
		end,
		info = "Deaths Advance",
	},
	--  Death Pact if less than 50 percent health
	deathpact50 = {
		abil = aL.DeathPact,
		order = 0,
		OffCD = function()
			return DKStatus.HasPet and DKStatus.PerHealth < 50 and aL.DeathPact:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			UpdateResource(aL.DeathPact.Cost)
			DKStatus.PerHealth = DKStatus.PerHealth + 50
			DKStatus.Health = DKStatus.Health + .5*DKStatus.MaxHealth
			aL.DeathPact.Useable = false
		end,
		info = "Death Pact if less than 50 percent health and pet is out",
	},
	-- Death Siphon when health is below 25%
	deathsiph25 = {
		abil = aL.DeathSiphon,
		order = 0,
		OffCD = function() 
			return  DKStatus.PerHealth < 25 and CheckResource(aL.DeathAndDecay.Cost)
		end,
		UpdateStatus = function()
			DKStatus.PerHealth = DKStatus.PerHealth + 100*(22000/DKStatus.MaxHealth)
			DKStatus.Health = DKStatus.Health + 22000
		end,
		info = "Death Siphon when health is below 25%",
	},
	--  Death Grip
	dg = {
		abil = aL.DeathGrip,
		order = 0,
		OffCD = function()
			return aL.DeathGrip:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.DeathGrip.Useable = false
		end,
		info = "Death Grip, dont put it in your melee priority you baddie! :P",
	},
	-- Death and Decay
	dnd = {
		abil = aL.DeathAndDecay,
		order = 0,
		OffCD = function()
			return aL.DeathAndDecay:OffCD(DKStatus.Time) and CheckResource(aL.DeathAndDecay.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.DeathAndDecay.Cost)
			aL.DeathAndDecay.Useable = false
		end,
		info = "Death and Decay",
	},
	-- Titanic Strength Flask
	dpsflask = {
		abil = aL.StrFlaskItem,
		order = 0,
		OffCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return false
			end
			if not aL.StrFlaskItem.Useable then -- This checks if we used the flask on the previous action
				return false
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return false
			end
			if aL.StrFlaskBuff:GetBuffDur(DKStatus.Time) < 600 then -- less than 10 minutes left on it
				return true
			else
				return false
			end
		end,
		UpdateStatus = function()
			aL.StrFlaskItem.Useable = false	
		end,
		info = "Use Str Flask, Only checks out of combat and in raid. It warns if you dont have one or it has less than 10 minutes left",
	},
	-- Strength Food
	dpsfood = {
		abil = aL.Seafood,  -- This is the ability associated with this action
		order = 0,
		OffCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return false
			end
			if not aL.Seafood.Useable then -- This checks if used the flask on the previous action
				return false
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return false
			end
			if aL.StrFoodBuff:GetBuffDur(DKStatus.Time) < 600 then -- less than 10 minutes left on it
				return true
			else
				return false
			end
		end,
		UpdateStatus = function()
			aL.Seafood.Useable = false
		end,
		info = "Use Str Food, Only checks out of combat in raid . It warns if you dont have one or it has less than 10 minutes left",
	},
	-- Empowered Rune Weapon, use on Cooldown when the equivalent of more than 4 runes are on cooldown
	erw = {
		abil = aL.EmpowerRuneWeapon,
		order = 0,
		OffCD = function()
			if not aL.EmpowerRuneWeapon:OffCD(DKStatus.Time) then
				return false
			end
			runes = (max(DKStatus.B1,DKStatus.B2) + max(DKStatus.U1,DKStatus.U2) + max(DKStatus.F1,DKStatus.F2))
			if runes > 32 and DKStatus.RP < 60 then
				return true
			else
				return false
			end
		end,
		UpdateStatus = function()
			aL.EmpowerRuneWeapon.Useable = false
			UpdateResource(aL.EmpowerRuneWeapon.Cost)
			-- So Painful  :*( 
			for i = 1,3 do
				DKStatus[r1order] = 0
				DKStatus[r2order] = 0
				DKStatus.RuneCD[i] = 0
				DKStatus.DRuneCD[i] = 0
			end
			
			if DKStatus.NotDeath[1] and DKStatus.NotDeath[2] then --neither blood runes are death runes
				DKStatus.RuneCD[1] = 2
			elseif DKStatus.NotDeath[1] ~= DKStatus.NotDeath[2] then -- they are split (different)
				DKStatus.RuneCD[1] = 1
				DKStatus.DRuneCD[1] = 1
			else -- Both must be death runes then
				DKStatus.DRuneCD[1] = 2
			end
			if DKStatus.NotDeath[3] and DKStatus.NotDeath[4] then --neither unholy runes are death runes
				DKStatus.RuneCD[2] = 2
			elseif DKStatus.NotDeath[3] ~= DKStatus.NotDeath[4] then -- they are split (different)
				DKStatus.RuneCD[2] = 1
				DKStatus.DRuneCD[2] = 1
			else -- Both must be death runes then
				DKStatus.DRuneCD[2] = 2
			end
			if DKStatus.NotDeath[5] and DKStatus.NotDeath[6] then --neither blood runes are death runes
				DKStatus.RuneCD[3] = 2
			elseif DKStatus.NotDeath[5] ~= DKStatus.NotDeath[6] then -- they are split (different)
				DKStatus.RuneCD[3] = 1
				DKStatus.DRuneCD[3] = 1
			else -- Both must be death runes then
				DKStatus.DRuneCD[3] = 2
			end
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[1] + DKStatus.DRuneCD[2] + DKStatus.DRuneCD[3]
		end,
		info = "Empowered Rune Weapon, use on Cooldown when the equivalent of more than 4 runes are on cooldown and rp is less than 60",
	},
	-- Golemblood Potion
	golem = {
		abil = aL.GolembloodItem, 
		order = 0,
		OffCD = function()		
			return aL.GolembloodItem:OffCD(DKStatus.Time) and aL.GolembloodItem.Useable and DKStatus.TargetIsBoss and DKStatus.BL > 10 
		end,
		UpdateStatus = function()
			aL.GolembloodItem.Useable = false  	
		end,
		info = "Golemblood potion only in raid on a boss when bloodlust is up",
	},
	-- Horn of Winter
	how = {
		abil = aL.HornOfWinter,
		order = 0,
		OffCD = function()
			return aL.HornOfWinter:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			UpdateResource(aL.HornOfWinter.Cost)
			DKStatus.APBuff = true
			aL.HornOfWinter.Useable = false
		end,
		info = "Horn of Winter",
	},
	-- Use Horn of Winter if missing str buff
	howbuff = {
		abil = aL.HornOfWinter,
		order = 0,
		OffCD = function()
			if DKStatus.APBuff then
				return false
			else
				return aL.HornOfWinter:OffCD(DKStatus.Time)
			end
		end,
		UpdateStatus = function()
			UpdateResource(aL.HornOfWinter.Cost)
			DKStatus.APBuff = true
			aL.HornOfWinter.Useable = false
		end,
		info = "Horn of Winter, if missing AP buff",
	},
	--  Icebound Fortitude
	icef = {
		abil = aL.IceboundFortitude,
		order = 0,
		OffCD = function()
			return aL.IceboundFortitude:OffCD(DKStatus.Time) and CheckResource(aL.IceboundFortitude.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.IceboundFortitude.Cost)
			aL.IceboundFortitude.Useable = false
		end,
		info = "Icebound Fortitude",
	},
	-- Icy Touch to apply frost fever
	itFF = {
		abil = aL.IcyTouch,
		order = 0,
		OffCD = function()
			return DKStatus.FF <= 1.5 and CheckResource(aL.IcyTouch.Cost) and aL.Outbreak:GetCD(DKStatus.Time) > 3
		end,
		UpdateStatus = function()
			UpdateResource(aL.IcyTouch.Cost)
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.ASDebuff = true
		end,
		info = "Icy Touch to apply frost fever if Outbreak won't be off CD shortly",
	},
	--  Lichborne
	lb = {
		abil = aL.Lichborne,
		order = 0,
		OffCD = function()
			return DKPlayerStats.Lichborne and aL.Lichborne:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.Lichborne.Useable = false
		end,
		info = "Lichborne",
	},
	--  Lichborne
	lbsh = {
		abil = aL.Lichborne,
		order = 0,
		OffCD = function()
			return DKPlayerStats.Lichborne and aL.Lichborne:OffCD(DKStatus.Time) and DKStatus.RP >= 80 and DKStatus.PerHealth < 50
		end,
		UpdateStatus = function()
			aL.Lichborne.Useable = false
		end,
		info = "Lichborne if off CD, runic power is more than 80 and health is less than 50%",
	},
	-- Use outbreak to refresh frost fever or blood plague
	out = {
		abil = aL.Outbreak,
		order = 0,
		OffCD = function()
			return (DKStatus.FF <= 2 or DKStatus.BP <=2) and aL.Outbreak:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.BP  = aL.BloodPlague.Lasts
			if DKPlayerStats.Spec == 1 then
				DKStatus.PDRDebuff = true
			end
			DKStatus.ASDebuff = true
			aL.Outbreak.Useable = false
		end,
		info = "Use Outbreak to refresh frost fever and/or blood plague",
	},
	-- Pestilence
	pes = {
		abil = aL.Pestilence,
		order = 0,
		OffCD = function()
			return DKStatus.FF > 0 and DKStatus.BP > 0 and CheckResource(aL.Pestilence.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.Pestilence.Cost)
		end,
		info = "Pestilence if both diseases are up, dont really want to use this, since there is nothing that indicates that it shouldnt be spammed, i.e no criteria as when to not use it",
	},
	-- Use Plague Leech when frost fever and blood plague are about to fall off
	pl = {
		abil = aL.PlagueLeech,
		order = 0,
		OffCD = function()
			return (DKStatus.FF <= 3 or DKStatus.BP <=3) and (DKStatus.FF > 0 and DKStatus.BP > 0) and aL.PlagueLeech:OffCD(DKStatus.Time) and ((DKStatus.B1 > 0 and DKStatus.B2 > 0) or  (DKStatus.U1 > 0 and DKStatus.U2 > 0) or (DKStatus.F1 > 0 and DKStatus.F2 > 0))
		end,
		UpdateStatus = function()
			DKStatus.FF  = 0
			DKStatus.BP  = 0
			DKStatus.PDRDebuff = false
			DKStatus.ASDebuff = false
			aL.PlagueLeech.Useable = false
		end,
		info = "Use Plague Leech when frost fever and blood plague are about to fall off,  work in progress",
	},
	-- Use Raise Dead on CD
	rd = {
		abil = aL.RaiseDead,
		order = 0,
		OffCD = function()
			return not DKStatus.HasPet and aL.RaiseDead:OffCD(DKStatus.Time) 
		end,
		UpdateStatus = function()
			aL.RaiseDead.Useable = false
		end,
		info = "Use Raise Dead on CD",
	},
	
	-- Use Unholy Blight to refresh frost fever or blood plague
	ublight = {
		abil = aL.UnholyBlight,
		order = 0,
		OffCD = function()
			return (DKStatus.FF <= 2 or DKStatus.BP <=2) and aL.UnholyBlight:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.BP  = aL.BloodPlague.Lasts
			if DKPlayerStats.Spec == 1 then
				DKStatus.PDRDebuff = true
			end
			DKStatus.ASDebuff = true
			aL.UnholyBlight.Useable = false
		end,
		info = "Use Unholy Blight to refresh frost fever and/or blood plague",
	},
	
---------------BLOOD------------------------------
	-- Bone Shield when stack are 0 and its off cooldown
	bone = {
		abil = aL.BoneShield,
		order = 1,
		OffCD = function() 
			if aL.BoneShield:GetStacks() > 0 then
				return false 
			else
				return aL.BoneShield:OffCD(DKStatus.Time) and CheckResource(aL.BoneShield.Cost)
			end
		end,
		UpdateStatus = function()
			UpdateResource(aL.BoneShield.Cost)
			aL.BoneShield.Useable = false
		end,
		info = "Bone Shield when stacks are 0 and its off cooldown",
	},
	-- Apply Blood Plague using Plague Strike to Debuff a tank target with scarlet fever
	bpdebuff = {
		abil = aL.PlagueStrike,
		order = 1,
		OffCD = function()
			return not DKStatus.PDRDebuff and CheckResource(aL.PlagueStrike.Cost) and aL.Outbreak:GetCD(DKStatus.Time) > 3 and DKPlayerStats.Spec == 1
		end,
		UpdateStatus = function()
			UpdateResource(aL.PlagueStrike.Cost)
			DKStatus.BP  = aL.BloodPlague.Lasts
			-- We checked that player was glyphed when we returned OffCD
			DKStatus.PDRDebuff = true
		end,
		info = "Debuff a tank target with scarlet fever, using Plague Strike if outbreak won't be off cooldown shortly, must be blood spec",
	},
	-- Blood Presence
	bpres = {
		abil = aL.BloodPresence,
		order = 1,
		OffCD = function()
			return DKStatus.Presence ~= 1
		end,
		UpdateStatus = function()
						
		end,
		info = "Make sure we are in Blood Presence",
	},
	--  Dancing Rune Weapon
	drw = {
		abil = aL.DancingRuneWeapon,
		order = 1,
		OffCD = function()
			return aL.DancingRuneWeapon:OffCD(DKStatus.Time) and CheckResource(aL.DancingRuneWeapon.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.DancingRuneWeapon.Cost)
			aL.DancingRuneWeapon.Useable = false
		end,
		info = "Dancing Rune Weapons on CD",
	},
	-- Death Strike
	ds = {
		abil = aL.DeathStrike,
		order = 1,
		OffCD = function()
			return CheckResource(aL.DeathStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.DeathStrike.Cost)
		end,
		info = "Death Strike",
	},
	-- Earthen Potion
	earthen = {
		abil = aL.EarthPotionItem, 
		order = 1,
		OffCD = function()		
			return aL.EarthPotionItem:OffCD(DKStatus.Time) and aL.EarthPotionItem.Useable and DKStatus.TargetIsBoss and DKStatus.TimeToDie < 60
		end,
		UpdateStatus = function()
			aL.EarthPotionItem.Useable = false 
		end,
		info = "Earthen potion, use only on a boss when ttd is less than 60 secs",
	},
	-- Heart Strike
	hs = {
		abil = aL.HeartStrike,
		order = 1,
		OffCD = function()
			return CheckResource(aL.HeartStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.HeartStrike.Cost)
		end,
		info = "Heart Strike",
	},
	-- Heart Strike only if we have 1.5 blood runes off CD or Blood Tap is off CD
	hs15 = {
		abil = aL.HeartStrike,
		order = 1,
		OffCD = function()
			return max(DKStatus.B1,DKStatus.B2) < 5
		end,
		UpdateStatus = function()
			UpdateResource(aL.HeartStrike.Cost)
		end,
		info = "Heart Strike only if we have 1.5 blood runes off CD or Blood Tap is off CD",
	},
	-- Icy Touch to debuff a tank target
	itdebuff = {
		abil = aL.IcyTouch,
		order = 1,
		OffCD = function()
			return not DKStatus.ASDebuff and CheckResource(aL.IcyTouch.Cost) and aL.Outbreak:GetCD(DKStatus.Time) > 3
		end,
		UpdateStatus = function()
			UpdateResource(aL.IcyTouch.Cost)
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.ASDebuff = true
		end,
		info = "Debuff a tank target with attack speed slow using Icy Touch if Outbreak won't be off CD shortly, only works correctly in blood spec",
	},
	-- Use outbreak to debuff a tank target
	outdebuff = {
		abil = aL.Outbreak,
		order = 1,
		OffCD = function()
			return (not DKStatus.ASDebuff or (not DKStatus.PDRDebuff and DKPlayerStats.Spec == 1)) and aL.Outbreak:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.BP  = aL.BloodPlague.Lasts
			if DKPlayerStats.Spec == 1 then
				DKStatus.PDRDebuff = true
			end
			DKStatus.ASDebuff = true
			aL.Outbreak.Useable = false
		end,
		info = "Use Outbreak to debuff a tank target, only works correctly in blood spec",
	},
	-- Rune Strike
	rs = {
		abil = aL.RuneStrike,
		order = 1,
		OffCD = function()
			return CheckResource(aL.RuneStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.RuneStrike.Cost)
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Rune Strike only useable in Blood Presence (doesnt track dodges or parries)",
	},
	-- Rune Strike if over 60 runic power
	rs60 = {
		abil = aL.RuneStrike,
		order = 1,
		OffCD = function()
			return DKStatus.RP >= 60
		end,
		UpdateStatus = function()
			UpdateResource(aL.RuneStrike.Cost)
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Rune Strike if 60 runic power or greater and in Blood Presence",
	},
	--  Rune Tap if less than 50 percent health
	rt50 = {
		abil = aL.RuneTap,
		order = 1,
		OffCD = function()
			return DKStatus.PerHealth < 50 and (DKStatus.WON or (aL.RuneTap:OffCD(DKStatus.Time) and CheckResource(aL.RuneTap.Cost)))
		end,
		UpdateStatus = function()
			if DKStatus.WON then
				DKStatus.WON = false
			else
				UpdateResource(aL.RuneTap.Cost)
			end
			DKStatus.PerHealth = DKStatus.PerHealth + 10
			DKStatus.Health = DKStatus.Health + .1*DKStatus.MaxHealth
			aL.RuneTap.Useable = false
		end,
		info = "Rune Tap when at less than 50 percent health",
	},
	--  Rune Tap if Will Of Necropolis procs and less than 75 percent health
	rt75won = {
		abil = aL.RuneTap,
		order = 1,
		OffCD = function()
			return DKStatus.PerHealth < 75 and DKStatus.WON
		end,
		UpdateStatus = function()
			DKStatus.PerHealth = DKStatus.PerHealth + 10
			DKStatus.Health = DKStatus.Health + .1*DKStatus.MaxHealth
			DKStatus.WON = false
			aL.RuneTap.Useable = false
		end,
		info = "Rune Tap if Will Of Necropolis procs when at less than 75 percent health",
	},
	-- Use Soul Reaper in blood spec when mobs health is below 35%
	srb = {
		abil = aL.SoulReaperB,
		order = 0,
		OffCD = function()
			return DKStatus.TargetPerHealth < 36 and aL.SoulReaperB:OffCD(DKStatus.Time) and CheckResource(aL.SoulReaperB.Cost)
		end,
		UpdateStatus = function()
			aL.SoulReaperB.Useable = false
		end,
		info = "Use Soul Reaper(Blood Spec) when mobs health is below 35%",
	},
	-- Stam Flask
	tankflask = {
		abil = aL.StamFlaskItem,  -- This is the ability associated with this action
		order = 1,
		OffCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return false
			end
			if not aL.StamFlaskItem.Useable then -- This checks if we used the flask on the previous action
				return false
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return false
			end
			if aL.StamFlaskBuff:GetBuffDur(DKStatus.Time) < 600 then -- less than 10 minutes left on it
				return true
			else
				return false
			end
		end,
		UpdateStatus = function()
			aL.StamFlaskItem.Useable = false	-- Toggle the cooldown on this ability that Im tracking so it 
									-- isn't use for the next ability
		end,
		info = "Use Stam Flask, Only checks out of combat and in raid. It warns if you dont have one or it has less than 10 minutes left",
	},
	-- Tank Food
	tankfood = {
		abil = aL.Seafood,  -- This is the ability associated with this action
		order = 1,
		OffCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return false
			end
			if not aL.Seafood.Useable then -- This checks if we used food on the previous action
				return false
			end
			if not DKPlayerStats.InRaid then -- Only suggest food in raids
				return false
			end
			if aL.DodgeFoodBuff:GetBuffDur(DKStatus.Time) + aL.MasteryFoodBuff:GetBuffDur(DKStatus.Time) + aL.ParryFoodBuff:GetBuffDur(DKStatus.Time) < 600 then -- We have buff
				return true
			else
				return false
			end
		end,
		UpdateStatus = function()
			aL.Seafood.Useable = false	-- Toggle the cooldown on this ability that Im tracking so it 
									-- isn't use for the next ability
		end,
		info = "Use Tank Food, Only checks out of combat and in raid. It warns if you dont have one or it has less than 10 minutes left",
	},
	--  Vampiric Blood when at half health or less
	vb50 = {
		abil = aL.VampiricBlood,
		order = 1,
		OffCD = function()
			return aL.VampiricBlood:OffCD(DKStatus.Time) and DKStatus.PerHealth < 50
		end,
		UpdateStatus = function()
			aL.VampiricBlood.Useable = false
		end,
		info = "Vampiric Blood when at half health or less",
	},
---------------FROST------------------------------
-- Blood Tap, use when you have 5 charges and KM or Pillar of Frost is up
	btKM = {
		abil = aL.BloodTap,
		order = 0,
		OffCD = function()
			return DKPlayerStats.BloodTap and DKStatus.BC >= 5 and (DKStatus.KM or aL.PillarOfFrost:OffCD(DKStatus.Time)) and ((DKStatus.B1 > 0 and DKStatus.B2 > 0) or  (DKStatus.U1 > 0 and DKStatus.U2 > 0) or (DKStatus.F1 > 0 and DKStatus.F2 > 0))
		end,
		UpdateStatus = function()
			-- In MoP, it always finishes a rune, which one it finishes depends on spec
			-- TODO
			--This is close but not right yet I dont think
			-- Blood Spec goes Blood -> Unholy -> Frost
			if DKStatus.Presence == 1 then
				if DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				end
			-- Frost Spec goes Unholy -> Blood -> Frost
			elseif DKStatus.Presence == 2 then
				if DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				elseif DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				end
			-- Unholy Spec goes Blood -> Frost -> Unholy
			elseif DKStatus.Presence == 3 then
				if DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 0 then
					if DKStatus.B1 > DKStatus.B2 then
						DKStatus.B1 = 0
					else
						DKStatus.B2 = 0
					end
					DKStatus.DRuneCD[1] = 1
				elseif DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 0 then
					if DKStatus.F1 > DKStatus.F2 then
						DKStatus.F1 = 0
					else
						DKStatus.F2 = 0
					end
					DKStatus.DRuneCD[3] = 1
				elseif DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 0 then
					if DKStatus.U1 > DKStatus.U2 then
						DKStatus.U1 = 0
					else
						DKStatus.U2 = 0
					end
					DKStatus.DRuneCD[2] = 1
				end
			end
			DKStatus.DRuneCD[4] = DKStatus.DRuneCD[4] + 1
			DKStatus.BC = DKStatus.BC - 5
		end,
		info = "Blood Tap, use when you have 5 charges and KM or Pillar of Frost is up",
	},
	-- Apply Frost Fever using Howling Blast
	ffHB = {
		abil = aL.HowlingBlast,
		order = 2,
		OffCD = function()
			if DKStatus.FF <= 1 then
				if DKStatus.Rime > 0 then
					return true
				else
					if CheckResource(aL.HowlingBlast.Cost) then
						return true
					end
				end
			end
			return false
		end,
		UpdateStatus = function()
			if DKStatus.Rime > 0 then
				DKStatus.Rime = DKStatus.Rime - 1
				UpdateResource(aL.Rime.Cost)
			else
				UpdateResource(aL.HowlingBlast.Cost)
			end
			DKStatus.FF  = aL.FrostFever.Lasts -- Update diseases
			DKStatus.ASDebuff = true
		end,
		info = "Apply frost fever using Howling Blast, glyphed frost dks only",
	},
	-- Frost Presence
	fpres = {
		abil = aL.FrostPresence,
		order = 2,
		OffCD = function()
			return DKStatus.Presence ~= 2
		end,
		UpdateStatus = function()
						
		end,
		info = "Frost Presence",
	},
	fs = {
		abil = aL.FrostStrike,
		order = 2,
		OffCD = function()
			return CheckResource(aL.FrostStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.FrostStrike.Cost)
			DKStatus.KM = false
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Frost Strike",
	},
	-- Frost strike if runic power is greater than 90
	fs90 = {
		abil = aL.FrostStrike,
		order = 2,
		OffCD = function()
			return DKStatus.RP >= 90
		end,
		UpdateStatus = function()
			UpdateResource(aL.FrostStrike.Cost)
			DKStatus.KM = false
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Frost Strike if runic power is 90 or greater",
	},
	-- Frost strike if runic power is greater than 60
	fs60 = {
		abil = aL.FrostStrike,
		order = 2,
		OffCD = function()
			return DKStatus.RP >= 60
		end,
		UpdateStatus = function()
			UpdateResource(aL.FrostStrike.Cost)
			DKStatus.KM = false
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Frost Strike if runic power is 60 or greater",
	},
	-- Howling Blast
	hb = {
		abil = aL.HowlingBlast,
		order = 2,
		OffCD = function()
			return DKStatus.Rime > 0 or CheckResource(aL.HowlingBlast.Cost)
		end,
		UpdateStatus = function()
			if DKStatus.Rime > 0 then
				DKStatus.Rime = DKStatus.Rime - 1
				UpdateResource(aL.Rime.Cost)
			else
				UpdateResource(aL.HowlingBlast.Cost)
			end
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.ASDebuff = true
		end,
		info = "Howling Blast",
	},
	-- Howling Blast if 1 each of blood and frost runes are off cooldown",
	hb11 = {
		abil = aL.HowlingBlast,
		order = 2,
		OffCD = function()
			return (DKStatus.DRuneCD[1] > 0) and (DKStatus.RuneCD[3] > 0) 
		end,
		UpdateStatus = function()
			if DKStatus.Rime > 0 then
				DKStatus.Rime = DKStatus.Rime - 1
				UpdateResource(aL.Rime.Cost)
			else
				UpdateResource(aL.HowlingBlast.Cost)
			end
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.ASDebuff = aL.FrostFever.Lasts
		end,
		info = "Howling Blast if both a blood and a frost rune are off cooldown",
	},
	-- Howling Blast if either both blood or both frost runes are on cooldown",
	hb2 = {
		abil = aL.HowlingBlast,
		order = 2,
		OffCD = function()
			return (DKStatus.RuneCD[3] == 2) or (DKStatus.DRuneCD[1] == 2) 
		end,
		UpdateStatus = function()
			if DKStatus.Rime > 0 then
				DKStatus.Rime = DKStatus.Rime - 1
				UpdateResource(aL.Rime.Cost)
			else
				UpdateResource(aL.HowlingBlast.Cost)
			end
			if DKPlayerStats.HowlingBlastGlyphed then
				DKStatus.FF  = aL.FrostFever.Lasts
				DKStatus.ASDebuff = true
			end
		end,
		info = "Howling Blast if either both blood or both frost runes are off cooldown",
	},
	-- Obliterate
	ob = {
		abil = aL.Obliterate,
		order = 2,
		OffCD = function()
			return CheckResource(aL.Obliterate.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.Obliterate.Cost)
			DKStatus.KM = false
		end,
		info = "Obliterate",
	},
	
	-- Obliterate if Killing machine procs
	obkm = {
		abil = aL.Obliterate,
		order = 2,
		OffCD = function()
			return DKStatus.KM and CheckResource(aL.Obliterate.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.Obliterate.Cost)
			DKStatus.KM = false
		end,
		info = "Obliterate if Killing machine procs",
	},
	-- Obliterate if one of each type is off CD
	ob111 = {
		abil = aL.Obliterate,
		order = 2,
		OffCD = function() -- Condition for usage implicitly checks if resources exist
			return (DKStatus.RuneCD[1] > 0 or DKStatus.DRuneCD[1] > 0) and (DKStatus.RuneCD[2] > 0 or DKStatus.DRuneCD[2] > 0) and (DKStatus.RuneCD[3] > 0 or DKStatus.DRuneCD[3] > 0)
		end,
		UpdateStatus = function()
			UpdateResource(aL.Obliterate.Cost)
			DKStatus.KM = false
		end,
		info = "Obliterate if 1 of each type of rune is off CD",
	},
	-- Obliterate if 1 pair of runes are completetly off CD
	ob2 = {
		abil = aL.Obliterate,
		order = 2,
		OffCD = function()
			return  CheckResource(aL.Obliterate.Cost) and ((DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 2) or (DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 2) or (DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 2))
			-- Kind of ugly conditions, have to check resources if pair isnt blood/death runes
		end,
		UpdateStatus = function()
			UpdateResource(aL.Obliterate.Cost)
			DKStatus.KM = false
		end,
		info = "Obliterate if 1 pair of runes are completetly off CD",
	},
	-- Obliterate if 2 pairs of runes are completetly off CD
	ob22 = {
		abil = aL.Obliterate,
		order = 2,
		OffCD = function()
			return ((DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 2) and (DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 2)) or ((DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 2) and (DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 2)) or  ((DKStatus.RuneCD[3] + DKStatus.DRuneCD[3] == 2) and (DKStatus.RuneCD[1] + DKStatus.DRuneCD[1] == 2))
		end,
		UpdateStatus = function()
			UpdateResource(aL.Obliterate.Cost)
			DKStatus.KM = false
		end,
		info = "Obliterate if 2 pairs of runes are completetly off CD",
	},
	-- Obliterate if 1 unholy rune is off cd and the other will be within 1 gcd
	ob2u = {
		abil = aL.Obliterate,
		order = 2,
		OffCD = function()
			return  CheckResource(aL.Obliterate.Cost) and ((DKStatus.U1 == 0  and DKStatus.U2 < 1.5) or (DKStatus.U2 == 0  and DKStatus.U1 < 1.5))
		end,
		UpdateStatus = function()
			UpdateResource(aL.Obliterate.Cost)
			DKStatus.KM = false
		end,
		info = "Obliterate if 1 unholy rune is off cd and the other will be very soon",
	},
	-- Pillar of Frost 
	pil = {
		abil = aL.PillarOfFrost,
		order = 2,
		OffCD = function()
			return CheckResource(aL.PillarOfFrost.Cost) and aL.PillarOfFrost:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			UpdateResource(aL.PillarOfFrost.Cost)
			aL.PillarOfFrost.Useable = false
		end,
		info = "Use Pillar of Frost on CD",
	},
	-- Howling Blast only when rime procs
	rime = {
		abil = aL.HowlingBlast,
		order = 2,
		OffCD = function()
			return DKStatus.Rime > 0 
		end,
		UpdateStatus = function()
			DKStatus.Rime = DKStatus.Rime - 1
			UpdateResource(aL.Rime.Cost)
			DKStatus.FF  = aL.FrostFever.Lasts
			DKStatus.ASDebuff = true
		end,
		info = "Howling Blast if Rime has procced",
	},
	-- Use Soul Reaper in Frost spec when mobs health is below 35%
	srf = {
		abil = aL.SoulReaperF,
		order = 0,
		OffCD = function()
			return DKStatus.TargetPerHealth < 36 and aL.SoulReaperF:OffCD(DKStatus.Time) and CheckResource(aL.SoulReaperF.Cost)
		end,
		UpdateStatus = function()
			aL.SoulReaperF.Useable = false
		end,
		info = "Use Soul Reaper(Frost Spec) when mobs health is below 35%",
	},
---------------UNHOLY-----------------------------
	-- Deathcoil if Runic Power is greater than 90
	dc90 = {
		abil = aL.DeathCoil,
		order = 3,
		OffCD = function() 
			return DKStatus.RP > 90
		end,
		UpdateStatus = function()
			if DKPlayerStats.Spec == 3 then
				DKStatus.SI = min(5, DKStatus.SI + 1)
			end
			if DKStatus.Doom > 0 then
				DKStatus.Doom = DKStatus.Doom - 1
			else
				UpdateResource(aL.DeathCoil.Cost)
			end
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Deathcoil if Runic Power is greater than 90",
	},
	-- Deathcoil if you have enough RP and Sudden Doom procs to create 5 stacks of SI
	dcnohulk = {
		abil = aL.DeathCoil,
		order = 3,
		OffCD = function() 
			return DKStatus.Hulk == 0 and DKStatus.SI < 5 and (floor(DKStatus.RP/aL.DeathCoil.Cost[5]) +  DKStatus.Doom >= (5 - DKStatus.SI))
		end,
		UpdateStatus = function()
			DKStatus.SI = min(5, DKStatus.SI + 1)
			if DKStatus.Doom > 0 then
				DKStatus.Doom = DKStatus.Doom - 1
			else
				UpdateResource(aL.DeathCoil.Cost)
			end
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Deathcoil if you have enough RP and Sudden Doom procs to create 5 stacks of SI"
	},
	-- Deathcoil if ams is up 
	dcams = {
		abil = aL.DeathCoil,
		order = 3,
		OffCD = function() 
			return DKStatus.RP > aL.DeathCoil.Cost[5] and aL.AntiMagicShell:GetBuff()
		end,
		UpdateStatus = function()
			if DKPlayerStats.Spec == 3 then
				DKStatus.SI = min(5, DKStatus.SI + 1)
			end
			if DKStatus.Doom > 0 then
				DKStatus.Doom = DKStatus.Doom - 1
			else
				UpdateResource(aL.DeathCoil.Cost)
			end
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Spam Deathcoil if ams is up, doesn't detect if you are actually soaking damage"
	},
	-- Death and Decay if 2 Unholy runes up and Runic Power is less than 110
	dnd2110 = {
		abil = aL.DeathAndDecay,
		order = 3,
		OffCD = function()
			return aL.DeathAndDecay:OffCD(DKStatus.Time) and DKStatus.RP < 110 and (DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 2)
		end,
		UpdateStatus = function()
			UpdateResource(aL.DeathAndDecay.Cost)
			aL.DeathAndDecay.Useable = false
		end,
		info = "Death and Decay if 2 Unholy runes are off CD and Runic Power is less than 110",
	},
	-- Deathcoil if we have a Sudden Doom Proc
	doom = {
		abil = aL.DeathCoil,
		order = 3,
		OffCD = function() 
			return DKStatus.Doom > 0 and not( DKStatus.Hulk ~= 0 and DKStatus.Hulk < 8 and DKStatus.RP < 100) 
		end,
		UpdateStatus = function()
			DKStatus.SI = min(5, DKStatus.SI + 1)
			DKStatus.Doom = DKStatus.Doom - 1
			if DKPlayerStats.BloodTap then
				DKStatus.BC = min(12,(DKStatus.BC + 2))
			end
		end,
		info = "Deathcoil if we have a Sudden Doom Proc and we aren't pooling for end of Dark Transformation",
	},
	-- dark transformation
	dt = {
		abil = aL.DarkTransformation,
		order = 3,
		OffCD = function()
			return DKStatus.SI == 5 and CheckResource(aL.DarkTransformation.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.DarkTransformation.Cost)
			DKStatus.SI  = 0
			DKStatus.Hulk = 30
		end,
		info = "Dark Transformation",
	},
	-- Festering Strike
	fes = {
		abil = aL.FesteringStrike,
		order = 3,
		OffCD = function() 
			return CheckResource(aL.FesteringStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.FesteringStrike.Cost) 
			if DKStatus.FF > 0  and DKStatus.FF < 4 then
				DKStatus.FF  = DKStatus.FF + 8
			end
			if DKStatus.BP > 0 and DKStatus.BP < 4 then
				DKStatus.BP  = DKStatus.BP + 8
			end
		end,			
		info = "Festering Strike",
	},
	-- Festering Strike if 2 blood and 2 frost runes off CD and Runic Power is less than 110
	fes22110 = {
		abil = aL.FesteringStrike,
		order = 3,
		OffCD = function() 
			return DKStatus.RP < 110 and (DKStatus.RuneCD[1] + DKStatus.RuneCD[3] == 4)
		end,
		UpdateStatus = function()
			UpdateResource(aL.FesteringStrike.Cost) -- these will eventually regen as death runes but we will update before that
			if DKStatus.FF > 0  and DKStatus.FF < 4 then
				DKStatus.FF  = DKStatus.FF + 8
			end
			if DKStatus.BP > 0 and DKStatus.BP < 4 then
				DKStatus.BP  = DKStatus.BP + 8
			end
		end,
		info = "Festering Strike if 2 blood and 2 frost runes off CD and Runic Power is less than 110",
	},
	-- Festering Strike if both blood and frost runes off CD and 1 pair is not death runes and Runic Power is less than 110
	fes11d11 = {
		abil = aL.FesteringStrike,
		order = 3,
		OffCD = function() 
			return DKStatus.RP < 110 and (DKStatus.RuneCD[1] == 1 and DKStatus.RuneCD[3] == 1 and DKStatus.DRuneCD[1] == 1 and DKStatus.DRuneCD[3] == 1)
		end,
		UpdateStatus = function()
			UpdateResource(aL.FesteringStrike.Cost) -- these will eventually regen as death runes but we will update before that
			if DKStatus.FF > 0  and DKStatus.FF < 4 then
				DKStatus.FF  = DKStatus.FF + 8
			end
			if DKStatus.BP > 0 and DKStatus.BP < 4 then
				DKStatus.BP  = DKStatus.BP + 8
			end
		end,
		info = "Festering Strike if both blood and frost runes off CD and 1 pair is not death runes and Runic Power is less than 110",
	},
	-- Festering Strike to refresh dieases if they are about to fall off
	fesFFBP = {
		abil = aL.FesteringStrike,
		order = 3,
		OffCD = function() 
			return ((0 < DKStatus.FF and DKStatus.FF < 3) or (0 < DKStatus.BP and DKStatus.BP < 3)) and (3 < aL.Outbreak:GetCD(DKStatus.Time) and aL.Outbreak:GetCD(DKStatus.Time) < 16) and CheckResource(aL.FesteringStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.FesteringStrike.Cost) 
			if DKStatus.FF > 0 and DKStatus.FF < 4 then
				DKStatus.FF  = DKStatus.FF + 8
			end
			if DKStatus.BP > 0 and DKStatus.BP < 4 then
				DKStatus.BP  = DKStatus.BP + 8
			end
		end,			
		info = "Festering Strike to extend dieases if they are about to fall off and Outbreak isnt off CD, but will be shortly"
	},
	-- Summon Gargoyle
	sg = {
		abil = aL.SummonGargoyle,
		order = 3,
		OffCD = function()
			return aL.SummonGargoyle:OffCD(DKStatus.Time) and CheckResource(aL.SummonGargoyle.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.SummonGargoyle.Cost)
			aL.SummonGargoyle.Useable = false
		end,
		info = "Summon Gargoyle on CD",
	},
	-- Use Soul Reaper in Unholy spec when mobs health is below 35%
	sru = {
		abil = aL.SoulReaperU,
		order = 0,
		OffCD = function()
			return DKStatus.TargetPerHealth < 36 and aL.SoulReaperU:OffCD(DKStatus.Time) and CheckResource(aL.SoulReaperU.Cost)
		end,
		UpdateStatus = function()
			aL.SoulReaperU.Useable = false
		end,
		info = "Use Soul Reaper(Unholy Spec) when mobs health is below 35%",
	},
	-- Scourge Strike 
	ss = {
		abil = aL.ScourgeStrike,
		order = 3,
		OffCD = function() 
			return CheckResource(aL.ScourgeStrike.Cost)
		end,
		UpdateStatus = function()
			UpdateResource(aL.ScourgeStrike.Cost)
		end,
		info = "Scourge Strike",
	},
	-- Scourge Strike if 2 Unholy runes off CD and Runic Power is less than 110
	ss2110 = {
		abil = aL.ScourgeStrike,
		order = 3,
		OffCD = function() 
			return DKStatus.RP < 110 and (DKStatus.RuneCD[2] + DKStatus.DRuneCD[2] == 2)
		end,
		UpdateStatus = function()
			UpdateResource(aL.ScourgeStrike.Cost)
		end,
		info = "Scourge Strike if 2 Unholy runes are off CD and Runic Power is less than 110",
	},
	-- Unholy Frenzy on CD
	uf = {
		abil = aL.UnholyFrenzy,
		order = 3,
		OffCD = function()
			return DKStatus.BL == 0 and aL.UnholyFrenzy:OffCD(DKStatus.Time)
		end,
		UpdateStatus = function()
			aL.UnholyFrenzy.Useable = false
		end,
		info = "Unholy Frenzy on CD",
	},
	-- Unholy Presence
	upres = {
		abil = aL.UnholyPresence,
		order = 3,
		OffCD = function()
			return DKStatus.Presence ~= 3
		end,
		UpdateStatus = function()
					
		end,
		info = "Unholy Presence",
	},
	none = {
		abil = aL.NoSpell,
		order = 0,
		OffCD = function()
			return true	
		end,
		UpdateStatus = function()
			UpdateResource(aL.NoSpell.Cost)
		end,
		info = "Do Nothing, used automatically when nothing in the priority list is off CD",
	},
}

-- List of actions that can be use in the Cooldown queue DKQCD
-- actionsCD, dont account for necessary resources instead they provide
-- the cooldown in secs subject to the specified conditions
-- 999 is the cooldown provided when the conditions arent met
local actionsCD = {
---------------ANY SPEC---------------------------
	--  Anti Magic Shell
	ams = {
		abil = aL.AntiMagicShell,
		order = 0,
		GetCD = function()
			return aL.AntiMagicShell:GetCD(DKStatus.Time)
		end,
		info = "Anti Magic Shell",
	},
	--  Anti Magic Zone
	amz = {
		abil = aL.AntiMagicZone,
		order = 0,
		GetCD = function()
			return aL.AntiMagicZone:GetCD(DKStatus.Time)
		end,
		info = "Anti Magic Zone",
	},
	-- Army of the dead
	army = {
		abil = aL.ArmyOfTheDead,
		order = 0,
		GetCD = function()
			runes = min(DKStatus.RuneCD[1],1) + min(DKStatus.RuneCD[2],1) + min(DKStatus.RuneCD[3],1) + DKStatus.DRuneCD[4]
			cd, start, duration = aL.ArmyOfTheDead:GetCD(DKStatus.Time)
			if cd == 0 then
				if runes > 2 then
					return 0,0,0
				else 
					return (3-runes)*2, 0, 0
				end
			else
				return cd, start, duration
			end
		end,
		info = "Army of the Dead",
	},
	--  Asphyxiate
	asphyx = {
		abil = aL.Asphyxiate,
		order = 0,
		GetCD = function()
			return aL.Asphyxiate:GetCD(DKStatus.Time)
		end,
		info = "Asphyxiate",
	},
	-- Arcane Torrent
	at = {
		abil = aL.ArcaneTorrent,
		order = 0,
		GetCD = function()
			return aL.ArcaneTorrent:GetCD(DKStatus.Time)
		end,
		info = "Arcane Torrent",
	},
	-- Berserking
	ber = {
		abil = aL.Berserking,
		order = 0,
		GetCD = function()
			return aL.Berserking:GetCD(DKStatus.Time)
		end,
		info = "Berserking",
	},
	-- Blood Fury
	bf = {
		abil = aL.BloodFury,
		order = 0,
		GetCD = function()
			return aL.BloodFury:GetCD(DKStatus.Time)
		end,
		info = "Blood Fury",
	},
	--  Cataclysmic Gladiator's Badge of Victory, PVP trinket, resil with on use str
	bov = {
		abil = aL.BadgeOfVictory,
		order = 0,
		GetCD = function()
			return aL.BadgeOfVictory:GetCD(DKStatus.Time)
		end,
		info = "Cataclysmic Gladiator's Badge of Victory, PVP trinket, resil with on use str, doesnt check if equipped",
	},
	-- Blood Tap Usage, use on Cooldown when more than 6 secs left on one of the 2 blood runes
	bt = {
		abil = aL.BloodTap,
		order = 0,
		GetCD = function()
			btcd, start, duration = aL.BloodTap:GetCD(DKStatus.Time)
			if btcd > 0 then
				return btcd, start, duration
			elseif DKStatus.B1 == 0 and DKStatus.B2 == 0 then
				runecd = 2
			elseif DKStatus.B1 == 0 then
				runecd = DKStatus.B2
			elseif DKStatus.B2 == 0 then
				runecd = DKStatus.B1
			else
				runecd =  min(DKStatus.B1, DKStatus.B2)
			end
			if runecd > 6 then
				return 0, 0, 0
			else
				return runecd, 0, 0
			end
		end,
		info = "Blood Tap, use on CD when more than 6 secs on rune refresh",
	},
	-- Dark Simulacrum on Cd
	darks = {
		abil = aL.DarkSimulacrum,
		order = 0,
		GetCD = function()
			return aL.DarkSimulacrum:GetCD(DKStatus.Time)
		end,
		info = "Dark Simulacrum",
	},
	-- Deaths Advance when at range
	da = {
		abil = aL.DeathsAdvance,
		order = 0,
		GetCD = function()
			if DKStatus.Melee then
				return 999, 0, 0
			else
				return aL.DeathsAdvance:GetCD(DKStatus.Time)
			end
		end,
		info = "Deaths Advance when at range",
	},
	--  Desecrated Ground
	desec = {
		abil = aL.DesecratedGround,
		order = 0,
		GetCD = function()
			return aL.DesecratedGround:GetCD(DKStatus.Time)
		end,
		info = "Desecrated Ground",
	},
	--  Death Grip
	dg = {
		abil = aL.DeathGrip,
		order = 0,
		GetCD = function()
			return aL.DeathGrip:GetCD(DKStatus.Time)
		end,
		info = "Death Grip",
	},
	-- Titanic Strength Flask
	dpsflask = {
		abil = aL.StrFlaskItem,  -- This is the ability associated with this action
		order = 0,
		GetCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return 999, 0, 0
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return 999, 0, 0
			end
			if aL.StrFlaskBuff:GetBuffDur(DKStatus.Time) < 600 then
				return 0, 0 , 0 
			else
				return 999, 0, 0
			end
		end,
		info = "Use Str Flask, Only checks out of combat and warns if you dont have one or it has less than 10 minutes left",
	},
	-- Strength Food
	dpsfood = {
		abil = aL.Seafood,  -- This is the ability associated with this action
		order = 0,
		GetCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return 999, 0, 0
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return 999, 0, 0
			end
			if aL.StrFoodBuff:GetBuffDur(DKStatus.Time) < 600 then
				return 0, 0, 0
			else
				return 999, 0, 0
			end
		end,
		info = "Use Str Food, Only checks out of combat and warns if you dont have one or it has less than 10 minutes left",
	},
	-- Empowered Rune Weapon, use on Cooldown when the equivalent of more 5 runes are on cooldown
	erw = {
		abil = aL.EmpowerRuneWeapon,
		order = 0,
		GetCD = function()
			erwcd, start, duration = aL.EmpowerRuneWeapon:GetCD(DKStatus.Time)
			if erwcd > 0 then
				return erwcd, start, duration
			else
				runes = (max(DKStatus.B1,DKStatus.B2) + max(DKStatus.U1,DKStatus.U2) + max(DKStatus.F1,DKStatus.F2))
				if runes > 32 and DKStatus.RP < 60 then
					return 0, 0, 0
				else
					return ((48 - runes)/6 + DKStatus.RP/24), 0, 0
				end
			end
		end,
		info = "Empowered Rune Weapon, use on Cooldown when the equivalent of more 4 runes are on cooldown",
	}, 
	-- Use Golemblood pot if bloodlust or time to die less than 60 secs and target is a boss
	golem = {
		abil = aL.GolembloodItem,
		order = 0,
		GetCD = function()
			if not DKStatus.TargetIsBoss then -- Only use pots on bosses
				return 999, 0, 0
			end
			if aL.GolembloodItem:OffCD(DKStatus.Time) and (DKStatus.BL > 10 or DKStatus.TimeToDie < 60) and DKStatus.TargetIsBoss then 
				return 0, 0, 0
			elseif DKStatus.TimeToDie < 600 then
				return max(aL.GolembloodItem:GetCD(DKStatus.Time), DKStatus.TimeToDie - 60, 0), 0, 0
			else
				return 999, 0, 0
			end
		end,
		info = "Use Golemblood pot if bloodlust or time to die is less than 60 secs and target is a boss",
	},
	--  Gorefiend's Grasp
	gg = {
		abil = aL.GorefiendsGrasp,
		order = 0,
		GetCD = function()
			return aL.GorefiendsGrasp:GetCD(DKStatus.Time)
		end,
		info = "Gorefiend's Grasp",
	},
	-- Gift Of Naaru when health is less than 50%
	gotn = {
		abil = aL.GiftOfNaaru,
		order = 0,
		GetCD = function()
			if not aL.GiftOfNaaru:OffCD(DKStatus.Time) then  
				return aL.GiftOfNaaru:GetCD(DKStatus.Time)
			elseif DKStatus.PerHealth < 50 then
				return 0, 0, 0
			else
				return DKStatus.PerHealth/20, 0, 0
			end
		end,
		info = "Gift Of Naaru when health is less than 50%",
	},
	--  Lichborne
	lb = {
		abil = aL.Lichborne,
		order = 0,
		GetCD = function()
			return aL.Lichborne:GetCD(DKStatus.Time)
		end,
		info = "Lichborne",
	},
	--  Malevolent Gladiator's Badge of Victory, dps trinket str with on use str
	malv = {
		abil = aL.MalvBadge,
		order = 0,
		GetCD = function()
			return aL.MalvBadge:GetCD(DKStatus.Time)
		end,
		info = "Malevolent Gladiator's Badge of Victory, dps trinket str with on use str",
	},
	--  Death Pact if less than 50 percent health
	pact50 = {
		abil = aL.DeathPact,
		order = 0,
		GetCD = function()
			if DKStatus.HasPet and DKStatus.PerHealth < 50 and aL.DeathPact:OffCD(DKStatus.Time) then
				return 0, 0, 0
			elseif aL.DeathPact:OffCD(DKStatus.Time) and (DKStatus.HasPet or aL.RaiseDead:OffCD(DKStatus.Time)) then
				return DKStatus.PerHealth/20, 0, 0
			elseif not DKStatus.HasPet then
				return max(aL.DeathPact:GetCD(DKStatus.Time), aL.RaiseDead:GetCD(DKStatus.Time), DKStatus.PerHealth/20, 12), 0, 0
			else
				return aL.DeathPact:GetCD(DKStatus.Time)
			end
		end,
		info = "Death Pact if less than 50 percent health",
	},
	-- Rocket Barrage
	rb = {
		abil = aL.RocketBarrage,
		order = 0,
		GetCD = function()
			return aL.RocketBarrage:GetCD(DKStatus.Time)
		end,
		info = "Rocket Barrage",
	},
	-- Raise Dead
	rd = {
		abil = aL.RaiseDead,
		order = 0,
		GetCD = function()
			if not DKStatus.HasPet then
				return aL.RaiseDead:GetCD(DKStatus.Time)
			else
				return 999, 0, 0
			end
		end,
		info = "Raise Dead",
	},
	--  Rotting Skull, Dps trinket, crit with on use str
	rotsk = {
		abil = aL.RottingSkull,
		order = 0,
		GetCD = function()
			return aL.RottingSkull:GetCD(DKStatus.Time)
		end,
		info = "Rotting Skull, Dps trinket, crit with on use str, doesnt check if equipped",
	},
	--  Remorseless Winter
	rw = {
		abil = aL.RemorselessWinter,
		order = 0,
		GetCD = function()
			return aL.RemorselessWinter:GetCD(DKStatus.Time)
		end,
		info = "Remorseless Winter",
	},
	-- Stoneform
	sf = {
		abil = aL.Stoneform,
		order = 0,
		GetCD = function()
			return aL.Stoneform:GetCD(DKStatus.Time)
		end,
		info = "Stoneform",
	},
---------------BLOOD---------------------------
	-- Bone Shield when stack are 0 and its off cooldown
	bone = {
		abil = aL.BoneShield,
		order = 1,
		GetCD = function() 
			if aL.BoneShield:GetStacks() > 0 then
				return 999, 0, 0
			else
				return aL.BoneShield:GetCD(DKStatus.Time)
			end
		end,
		info = "Bone Shield when stack are 0",
	},
	--  Dancing Rune Weapon
	drw = {
		abil = aL.DancingRuneWeapon,
		order = 1,
		GetCD = function()
			return aL.DancingRuneWeapon:GetCD(DKStatus.Time)
		end,
		info = "Dancing Rune Weapons",
	},
	-- Use Earthen pot if time to die less than 60 secs and target is a boss
	earthen = {
		abil = aL.EarthPotionItem,  -- This is the ability associated with this action
		order = 1,
		GetCD = function()
			if not DKStatus.TargetIsBoss then -- Only use pots on bosses
				return 999, 0, 0
			end
			if aL.EarthPotionItem:OffCD(DKStatus.Time) and DKStatus.TimeToDie < 60 and DKStatus.TargetIsBoss then
				return 0, 0, 0
			elseif DKStatus.TimeToDie < 600 then
				return max(aL.EarthPotionItem:GetCD(DKStatus.Time), DKStatus.TimeToDie - 60, 0), 0, 0
			else
				return 999, 0, 0
			end
		end,
		info = "Use Earthen pot if time to die is less than 60 secs and target is a boss",
	},
	--  Fire of the Deep, Tank trinket mastery with on use dodge
	fod = {
		abil = aL.FireOfTheDeep,
		order = 1,
		GetCD = function()
			return aL.FireOfTheDeep:GetCD(DKStatus.Time)
		end,
		info = "Fire of the Deep, Tank trinket mastery with on use dodge, doesnt check if equipped",
	},
	--  Icebound Fortitude
	icef = {
		abil = aL.IceboundFortitude,
		order = 1,
		GetCD = function()
			return aL.IceboundFortitude:GetCD(DKStatus.Time)
		end,
		info = "Icebound Fortitude",
	},
	--  Rune Tap
	rt = {
		abil = aL.RuneTap,
		order = 1,
		GetCD = function()
			return aL.RuneTap:GetCD(DKStatus.Time)
		end,
		info = "Rune Tap",
	},
	-- Stam Flask
	tankflask = {
		abil = aL.StamFlaskItem,  -- This is the ability associated with this action
		order = 1,
		GetCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return 999, 0, 0
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return 999, 0 , 0
			end
			if aL.StamFlaskBuff:GetBuffDur(DKStatus.Time) < 600 then
				return 0, 0, 0
			else
				return 999, 0, 0
			end
		end,
		info = "Use Stam Flask, Only checks out of combat and in raid. It warns if you dont have one or it has less than 10 minutes left",
	},
	-- Tank Food
	tankfood = {
		abil = aL.Seafood,  -- This is the ability associated with this action
		order = 1,
		GetCD = function()
			if DKStatus.InCombat then -- Just bail if we are in combat so we arent checking this constantly
				return 999, 0, 0
			end
			if not DKPlayerStats.InRaid then -- Only suggest flask in raids
				return 999, 0, 0
			end
			if aL.DodgeFoodBuff:GetBuffDur(DKStatus.Time) + aL.MasteryFoodBuff:GetBuffDur(DKStatus.Time) + aL.ParryFoodBuff:GetBuffDur(DKStatus.Time) < 600 then -- We have buff
				return 0, 0, 0
			else
				return 999, 0, 0
			end
		end,
		info = "Use Tank Food, Only checks out of combat in raid. It warns if you dont have one or it has less than 10 minutes left",
	},
	--  Vampiric Blood 
	vb = {
		abil = aL.VampiricBlood,
		order = 1,
		GetCD = function()
			return aL.VampiricBlood:GetCD(DKStatus.Time)
		end,
		info = "Vampiric Blood",
	},
---------------FROST---------------------------
	-- Pillar of Frost
	pil = {
		abil = aL.PillarOfFrost,
		order = 2,
		GetCD = function()
			return aL.PillarOfFrost:GetCD(DKStatus.Time)
		end,
		info = "Pillar of Frost",
	},
	-- Raise Dead with max attack power, this one is sort of hand crafted
	rdmax = {
		abil = aL.RaiseDead,
		order = 2,
		GetCD = function()
			cd, start, duration  = aL.RaiseDead:GetCD(DKStatus.Time)
			if cd > 0  then 
				-- Ghoul isnt off CD
				return cd, start, duration
			elseif DKPlayerStats.CrecheEquipped and DKPlayerStats.EyeEquipped then
				-- The buff duration from golem potion
				potion = aL.GolembloodBuff:GetBuffDur(DKStatus.Time)	
				eyestacks = aL.Eye:GetStacks() 
				cretrink = aL.Creche:GetBuffDur(DKStatus.Time) 
				-- This is actually checking the internal cd on creche which I track
				crecd = max(0, aL.Creche.CooldownLength + aL.Creche.Start - DKStatus.Time)
				-- Everything is up, Summon ghoul!
				if DKStatus.Pillar > 0 and DKStatus.RotFC > 0  and cretrink  > 0 and eyestacks == 10 then 
					return 0, 0, 0
				-- Everything is up but creche hasnt procced yet, although it's off Cd, wait for it to proc
				elseif DKStatus.Pillar > 0 and DKStatus.RotFC > 0 and potion > 0 and eyestacks == 10 and crecd == 0 then
					return  max(0,min(DKStatus.Pillar -3, DKStatus.RotFC -3, potion -3)), 0, 0 
				-- Creche isnt coming off Cd before we lose Pillar or RoFC, so just use immediately
				elseif DKStatus.Pillar > 0 and DKStatus.RotFC > 0 and eyestacks == 10 and (crecd > DKStatus.Pillar or crecd > DKStatus.RotFC) then 
					return  0, 0, 0
				-- Creche could come off CD and procc before we lose Pillar or RoFC so wait
				elseif DKStatus.Pillar > 0 and DKStatus.RotFC > 0 and eyestacks == 10 and (crecd < DKStatus.Pillar and crecd < DKStatus.RotFC) then
					return  max(0,min(DKStatus.Pillar -3, DKStatus.RotFC-3)), 0, 0
				-- We have Pillar but stupid RoFC hasnt procced, wait as long as we can
				elseif DKStatus.Pillar > 0 then
					return  max(DKStatus.Pillar-3,0), 0, 0
				-- At least wait for Pillar before we use ghoul
				elseif DKStatus.PillarCD > 0 then
					return  DKStatus.PillarCD , 0, 0
				-- Pillar is ready but for some reason we haven't used it yet
				else
					return  1 , 0, 0
				end
			else
				if DKStatus.Pillar > 0 and DKStatus.RotFC > 0 then 
					return 0, 0, 0
				-- We have Pillar but stupid RoFC hasnt procced, wait as long as we can
				elseif DKStatus.Pillar > 0 then
					return  max(DKStatus.Pillar-3,0), 0, 0
				-- At least wait for Pillar before we use ghoul
				elseif DKStatus.PillarCD > 0 then
					return  DKStatus.PillarCD , 0, 0
				else 
					return  1 , 0, 0
				end
			end
		end,
		info = "Raise Dead when Pillar and RotFC are up and hopefully when creche trinket procs",
	},
---------------UNHOLY--------------------------
	--  Summon Gargoyle
	sg = {
		abil = aL.SummonGargoyle,
		order = 3,
		GetCD = function()
			return aL.SummonGargoyle:GetCD(DKStatus.Time)
		end,
		info = "Summon Gargoyle",
	},
	--  Unholy Frenzy
	uf = {
		abil = aL.UnholyFrenzy,
		order = 3,
		GetCD = function()
			return aL.UnholyFrenzy:GetCD(DKStatus.Time)
		end,
		info = "Unholy Frenzy",
	},
	none = {
		abil = aL.NoSpell,
		order = 0,
		GetCD = function()
			return 997, 0, 0	
		end,
		info = "Do Nothing, internal action, no need to add, just creates a blank button",
	},
	none2 = {
		abil = aL.NoSpell,
		order = 0,
		GetCD = function()
			return 998, 0, 0	
		end,
		info = "Do Nothing Twice! internal action, no need to add, just creates a blank button",
	},
}
-----------
-- This global function just passes the complete action list by reference for use in other files
-- The options file specifically
function addonTable.DKOneActionList()
	return actions
end
function addonTable.DKOneActionCDList()
	return actionsCD
end

-- This takes the 4 text strings contained in DKPlayerStats.Rotation it parses them into 
-- an ordered array of actions, weeding out those that dont match the listed
-- actions or arent usable by the player
function addonTable.UpdateQueue(rotarg)
	-- We want to wipe previous queues so we dont just reappend actions to them
	wipe(DKQ)
	wipe(DKQAOE)
	wipe(DKQR)
	wipe(DKQCD)
	-- first we do the standard melee priority queue
	-- go through the string and pull out each space seprated word
	for v in string.gmatch(rotarg.prio, "[^ ]+") do
		-- check the word is a key in the table of all available actions
		-- and that the ability associated with that particular action
		-- is known to the player
		if actions[v] and actions[v].abil.Useable then
			tinsert(DKQ, v) -- we add the key to an array in order
		end
	end
	-- TODO Better way to let the user know they have made a mistake in their priority list
	
	-- We add an ability to the end of every queue corresponding to NoSpell
	-- NoSpell just shows a transparent button (with border), this corresponds
	-- to not having any avaible action in the priority queueu
	tinsert(DKQ, "none")
	
	-- AOE queue
	for v in string.gmatch(rotarg.prioAOE, "[^ ]+") do
		if actions[v] and actions[v].abil.Useable then
			tinsert(DKQAOE, v)
		end
	end
	tinsert(DKQAOE, "none")
		
	-- Range queue
	for v in string.gmatch(rotarg.prioR, "[^ ]+") do
		if actions[v] and actions[v].abil.Useable then
			tinsert(DKQR, v)
		end
	end
	tinsert(DKQR, "none")
		
	-- Cooldown queue
	for v in string.gmatch(rotarg.prioCD, "[^ ]+") do
		if actionsCD[v] and actionsCD[v].abil.Useable then
			tinsert(DKQCD, v)
		end
	end
	-- Since we cant have duplicate cooldowns on the 
	-- primary and secondary cooldown buttons we need
	-- 2 NoSpell cooldown actions
	tinsert(DKQCD, "none")
	tinsert(DKQCD, "none2")
end
-- Finds the first action in the priority queue that is off cooldown
local function GetNextAction()
	-- Each Spec has a different set of Priority Queues
	-- These include the default Queue, DKQ, the AOE Queue, DKQAOE, 
	-- and the ranged Queue, DKQR
	-- Here, based on the info given by the status variable
	-- we set the current queue to the appropriate one
	if DKStatus.AOE  then
		CurrentQ = DKQAOE
	elseif not DKStatus.Melee then
		CurrentQ = DKQR
	else
		CurrentQ = DKQ
	end
	n = #CurrentQ
	-- We step through the current queue one by one, checking if each action 
	-- in the queue is off cooldown, we return the first action we find off cooldown
	-- Since all the queues are anchored by NoSpell which has no CD this always returns something
	for i = 1, n do
		if actions[CurrentQ[i]].OffCD() then
			return CurrentQ[i]
		end
	end
end

local function GetNextActionCD()
	-- This returns the 2 actionsCD with the shortest CD and their CDs
	-- ties are returned in the priority order
	n = #DKQCD  --number of actions in cooldown priority list
	cd = 0
	cd1 = 1000
	cd2 = 1000
	start = 0
	start1 = 0
	start2 = 0
	duration = 0
	duration1 = 0
	duration2 = 0
	i1 = 1 -- indicies of smallest cooldown
	i2 = 1 -- indicies of 2nd smallest cooldown
		
	for i = 1, n do -- Step through the cooldown action priority list
		cd, start, duration = actionsCD[DKQCD[i]].GetCD() -- we find the current action in the priority list and get its cooldown
		if cd < cd1 then -- current actionCD has shortest CD so far
			i2 = i1 -- We keep track of the index
			i1 = i
			start2 = start1 -- start
			start1 = start
			duration2 = duration1 -- duration
			duration1 = duration
			cd2 = cd1 -- and cooldown
			cd1 = cd
		elseif cd < cd2 then -- current actionCD has 2nd shortest CD so far
			i2 = i
			start2 = start
			duration2 = duration
			cd2 = cd
		end
		if cd1 == 0 and cd2 == 0 then 	-- since we traverse the actionCDs in priority order
			break						-- once we find two off cooldown we are done
		end
	end
	-- Return the 2 cooldown actions and their cooldown info (so we dont have to recalculate it later
	return DKQCD[i1], DKQCD[i2], cd1, start1, duration1, cd2, start2, duration2
end

-- This function just updates all the status variables to a later time
-- Shouldnt bump more than the rotation of a rune at once  i.e. make sure TimeBump < DKSTatusC.RuneDur
local function UpdateStatusTime(TimeBump) 
	--RuneConCheck("Begining UpdateStatusTime")
	if TimeBump == 0 then -- Just bail out if the time is 0
		return
	end
	-- Subtract the time from each rune cooldown
	for i = 1,6 do -- Go through each rune type
		DKStatus[runeorder[i]] = max(0, DKStatus[runeorder[i]] - TimeBump)
	end
	-- recount runes of each type
	for i = 1,3 do -- Go through each rune type
		-- Intialize number of runes off CD to zero
		DKStatus.RuneCD[i] = 0
		DKStatus.DRuneCD[i] = 0
		-- First rune of type  1,3,5
		if DKStatus[r1order[i]] == 0 then -- 0 would indicate its off CD
			if DKStatus.NotDeath[2*i - 1] then -- increment the correct count
				DKStatus.RuneCD[i] = DKStatus.RuneCD[i] + 1
			else
				DKStatus.DRuneCD[i] = DKStatus.DRuneCD[i] + 1
			end
		end
		-- Second rune of type  2,4,6
		if DKStatus[r2order[i]] == 0 then -- 0 would indicate its off CD
			if DKStatus.NotDeath[2*i] then -- increment the correct count
				DKStatus.RuneCD[i] = DKStatus.RuneCD[i] + 1
			else
				DKStatus.DRuneCD[i] = DKStatus.DRuneCD[i] + 1
			end
		end
	end
	-- the 4th deathrune count is just a sum of all 3 types
	DKStatus.DRuneCD[4] = DKStatus.DRuneCD[1] + DKStatus.DRuneCD[2] + DKStatus.DRuneCD[3]
	-- Subtract time from disease timers
	DKStatus.FF = max(0, DKStatus.FF - TimeBump)
	DKStatus.BP = max(0, DKStatus.BP - TimeBump)
	-- Subtract Time from the GCDLeft
	DKStatus.GCDLeft = max(0, DKStatus.GCDLeft - TimeBump)
	-- Add to current time
	DKStatus.Time = DKStatus.Time + TimeBump
end

-- This is the primary function for determing the suggested actions
-- It's called from the DKOne file
function addonTable.Rotation()
	-- First we update the current status to what it will be when we come off the GCD
	-- So we can use runes that might be coming off CD before the GCD finishes for the
	-- the next action for example
	UpdateStatusTime(DKStatus.GCDLeft)
	-- Next we determine the highest priority action that will be available next
	action1 = GetNextAction()
	s1 = actions[action1].abil -- We get a reference to the ability associated with the current action
	-- We update status to account for changes caused by that action
	-- Primarily to resources like runes and runic power
	-- But also buffs and debuffs etc
	actions[action1]:UpdateStatus()
	-- We bump the time up to the next GCD after the action we choose in order to 
	-- plan the next action
	UpdateStatusTime(DKStatus.GCD)
	-- we get the next action
	action2 = GetNextAction()
	-- and find its associated ability
	s2 = actions[action2].abil
	-- As a work around we have a toggle which we use to indicate some action are unavailable for 
	-- the second action if they were used for the first action. We set that toggle back to true
	-- after calculating the second action
	actions[action1].abil.Useable = true
	return s1, s2
end

-- This is the primary function for determing the suggested Cooldown actions
-- It's called from the DKOne file
function addonTable.RotationCD()
	-- We find the 2 cooldown actions and thier associated cooldowns
	labelCD1, labelCD2, cd1, start1, duration1, cd2, start2, duration2  = GetNextActionCD()
	-- we find the abilities associated with those actions
	actionCD1 = actionsCD[labelCD1].abil
	actionCD2 = actionsCD[labelCD2].abil
	-- send it all back to to the OnUpdate() function in the DKOne file
	return actionCD1, actionCD2, cd1, start1, duration1, cd2, start2, duration2
end
