-- Additional file for the DKOne WOW addon
-- Author: Zorben on Proudmoore-US

-- Addon bails if character is not a DK
if select(2,UnitClass("player"))  ~= "DEATHKNIGHT" then 
	DisableAddOn("DKOne") 
	return
	
end
local addonName, addonTable = ... --Pulls back the Addon-Local Variables and stores them locally
local aL = addonTable.DKOneAbilityList()		-- local reference to the abilityList table
local DKStatus = addonTable.DKOneDKStatus()	-- local reference to the DKStatus table
local DKPlayerStats = addonTable.DKOneDKPlayerStats()	-- local reference to the DKPlayerStats table
-- We make a local reference since we call the database variable so frequently
local DB = DKOneDB
-- Color Order for different rune orders
local cOrder = {	{1,1,2,2,3,3},
					{1,1,3,3,2,2},
					{2,2,1,1,3,3},
					{2,2,3,3,1,1},
					{3,3,1,1,2,2},
					{3,3,2,2,1,1},
				}
-- The 6 different possible rune orders
local rOrder = {	{1,2,3,4,5,6},
					{1,2,5,6,3,4},
					{3,4,1,2,5,6},
					{5,6,1,2,3,4},
					{3,4,5,6,1,2},
					{5,6,3,4,1,2},
				}
-- 9 anchor points for regions				
local point = {
	"TOPLEFT",
	"TOP",
	"TOPRIGHT",
	"RIGHT",
	"BOTTOMRIGHT",
	"BOTTOM",
	"BOTTOMLEFT",
	"LEFT",
	"CENTER",
}

local pOrderH = {1, 3, 8, 4, 7, 5} -- Point Order when runes are Horizontal
local pOrderV = {7, 1, 6, 2, 5, 3} -- Point order when runes are vertical
local runeorder = {"B1", "B2", "U1", "U2", "F1", "F2"} -- Rune Order used in Blizzard API, remember BUF
local runeValues = {0,0,0,0,0,0} -- temp array holding the CD for each rune in the native order
local notDeathRunes = {true, true,true,true,true,true} -- temp array for indicating whether rune is death rune
local runeFlash = {0,0,0,0,0,0} -- Array of time stamps for whne runes came off CD for determining whether to flash or not
local flashTrue = {false, false, false, false, false, false} -- Should the rune show the lighter color indicating it just came off CD
local pad = 5 -- initialization value for space between runes
-- These arrays are used to place the runes in the rune frame
-- for both x and y position and whther its vertical or horizontal
local offsetxH = {pad, -pad, pad, -pad, pad, -pad}
local offsetyH = {-pad, -pad, 0, 0, pad, pad}
local offsetxV = {pad, pad, 0, 0, -pad, -pad}
local offsetyV = {pad, -pad, pad, -pad, pad, -pad}
local fontFace, fontFlags, _,  statusMax -- local variables used in redrawing labels
local rune1order = {1,3,5} -- just a helper array for indexing through 1st rune of each type
local rune2order = {2,4,6} -- for 2nd rune of each type
local textureB1, textureB2, textureB3, textureB4 -- stores the icons displayed on the 2 buttons
local sparkTex = "Interface\\AddOns\\DKOne\\textures\\lineblend"	
local sparkX = {0, 0, 0, 0} -- This is for the runic power divider lines for runic power dumps
local sparkHeight = 1 -- The height of the divider lines
local sparkWidth = 1 -- width of the divider lines
local sparkSpaceMultiplier = 1 -- multiply the RP cost times this to get the current spacing between lines (in pixels)
local rpWidth = 0 --This is for setting the Runic Power Width in redrawRuneFrame()
local rpHeight = 0 --This is for setting the Runic Power Height in redrawRuneFrame()
local bsWidth = 0 --This is for setting the Blood Shield Width in redrawRuneFrame() 
local bsHeight = 0 --This is for setting the Blood Shield Height in redrawRuneFrame()
local BSsparkXMultiplier = 1 -- Multiple Blood Shield Duration by this to get position of BS dura tion spark (in pixels)
-- Local function pointers
local floor = floor --Local version of library functions
local CreateFrame = CreateFrame --Local version of library functions
local unpack = unpack --Local version of library functions
local tostring = tostring --Local version of library functions
local format = format --Local version of library functions

function addonTable.createRuneFrame(parent)
	--Creators
	-- Create an individual status bar that will be a child of RuneFrame, used for runes, runic power, dots, and blood shield
	local function makeStatusBar(name, parent)
		local bar = CreateFrame("StatusBar", name, parent)
		--Interface\\TargetingFrame\\UI-StatusBar --alternative texture
		bar:SetStatusBarTexture("Interface\\AddOns\\DKOne\\textures\\Flat")--local texture, from recount originally I think
		bar:GetStatusBarTexture():SetHorizTile(false)
		bar:GetStatusBarTexture():SetVertTile(false)
		bar.bg = bar:CreateTexture(nil, "BACKGROUND")
		bar.bg:SetAllPoints(bar)
		bar.bg:SetTexture(unpack(DB.RuneFrame.colorRbg))
		bar.label = bar:CreateFontString("$parentCount", "OVERLAY", "TextStatusBarText")
		bar.labelprefix = "1"
		bar.labelsuffix = ""
		return bar
	end
	-- creat a button, used for buttons 1 and 2
	local function makeButton(name, parent)
		local button = CreateFrame("Button", name , parent)
		button:EnableMouse(false)
		button.texture = button:CreateTexture("$parentIcon", "BACKGROUND")
		button.texture:SetAllPoints()
		button.label = button:CreateFontString("$parentCount", "OVERLAY", "TextStatusBarText")
		return button
	end
	-- REDRAW these functions are called, when the user changes a configuration option, or changes specs
	-- redraws the button given all the configuration parameters
	local function redrawButton(self, button, size, texture, point, parent, pointParent, offsetx, offsety)
		button:SetWidth(size)
		button:SetHeight(size)
		button:SetPoint(point, parent, pointParent, offsetx, offsety)
		button.texture:SetTexture(texture)
	end
	-- redraws the labels on the buttons (stacks or duration)
	local function redrawButtonLabel(self, button, offsetx, offsety, fontsize, fontColor, show)
		fontFace, _, fontFlags = button.label:GetFont()
		button.label:SetFont(fontFace, fontsize, fontFlags)
		button.label:ClearAllPoints()
		button.label:SetPoint("CENTER", button, "CENTER", offsetx, offsety)
		button.label:SetTextColor(unpack(fontColor))
		if show then
			button.label:Show()
		else
			button.label:Hide()
		end
	end
	-- Since Rune durations change with haste values, this is a helper function to change the
	-- max value for the rune status bars, and update the text
	-- This function is not used at the moment. I am resetting the duration every update
	-- to ensure the max value is changed before value is set, otherwise
	-- the runes get jumpy looking when runic corruption is proccing frequently
	local function updateRunDur(self, runDur)
		for i = 1,6 do
			self.Runes[i]:SetMinMaxValues(0,runDur)
			self:setRuneLabelSuffix(self.Runes[i], runDur)
		end
	end
	-- Update the label suffix when we change the Duration on the rune bars
	local function setRuneLabelSuffix(self,bar,runDur)
		if DB.RuneFrame.formatRlabels == 3 then
			bar.labelsuffix = "/"..tostring(floor(runDur*10 + .5)/10)
		elseif DB.RuneFrame.formatRlabels == 4 then
			bar.labelsuffix = "/"..tostring(floor(runDur + .5))
		else
			bar.labelsuffix = ""
		end
	end
	
	-- Set all the user defined options for a statusbar
	local function redrawStatusBar(self, bar, width, height, minval, maxval, point, parent, pointParent, offsetx, offsety, color, colorbg, orientation)
		bar:ClearAllPoints()
		bar:SetPoint(point, parent, pointParent, offsetx, offsety)
		bar:SetWidth(width)
		bar:SetHeight(height)
		bar:SetStatusBarColor(unpack(color))
		bar:SetMinMaxValues(minval,maxval)
		bar:SetValue(.5)
		bar:SetOrientation(orientation)
		bar.bg:SetTexture(unpack(colorbg))
	end
	-- Set all the user defined options for the text on top of the statusbar
	local function redrawLabel(self, bar, offsetx, offsety, fontsize, show, labelformat)
		fontFace, _, fontFlags = bar.label:GetFont()
		bar.label:SetFont(fontFace, fontsize, fontFlags)
		bar.label:ClearAllPoints()
		bar.label:SetPoint("CENTER", bar, "CENTER", offsetx, offsety)
		_, statusMax = bar:GetMinMaxValues()
		-- The configuration  options alllow many text labels to include a "value/maxvalue" option for display
		-- the "/maxvalue" is the labelsuffix part
		if labelformat == 3 then
			bar.labelsuffix = "/"..tostring(floor(statusMax*10 + .5)/10)
		elseif labelformat == 4 then
			bar.labelsuffix = "/"..tostring(floor(statusMax + .5))
		else
			bar.labelsuffix = ""
		end
		bar.label:SetJustifyH("CENTER")
		if show then
			bar.label:Show()
		else
			bar.label:Hide()
		end
	end
	-- For frost specifically when switching into and out of frost presence
	local function redrawRuneSpark(self)
		sparkX[1] = (aL.FrostStrike.Cost[5]*sparkSpaceMultiplier)
		sparkX[2] = 2*sparkX[1]
		sparkX[3] = 3*sparkX[1]
		sparkX[4] = 4*sparkX[1]
		for i = 1,4 do
				self.RP.Spark[i]:ClearAllPoints()
				if DB.RuneFrame.orientRP == "HORIZONTAL" then
					self.RP.Spark[i]:SetPoint("CENTER", self.RP, "LEFT", sparkX[i], 0)
				else
					self.RP.Spark[i]:SetPoint("CENTER", self.RP, "BOTTOM", 0, sparkX[i])
				end
				self.RP.Spark[i]:SetHeight(sparkHeight)
				self.RP.Spark[i]:SetWidth(sparkWidth)
				self.RP.Spark[i]:Show()
				if DKStatus.Presence ~= 2 then
					if i > 2 then
						self.RP.Spark[i]:Hide()
					end
				end
		end
	end
	-- Set all the user defined options for the RuneFrame
	local function redrawRuneFrame(self)
		self.bg:SetTexture(unpack(DB.RuneFrame.colorRFbg)) --Background color
		self:SetPoint("CENTER", DB.RuneFrame.x, DB.RuneFrame.y) -- Location
		-- Spacing of the runes in the runeframe depends on the orientation of the runes
		pad = DB.RuneFrame.padding
		offsetxH = {pad, -pad, pad, -pad, pad, -pad}
		offsetyH = {-pad, -pad, 0, 0, pad, pad}
		offsetxV = {pad, pad, 0, 0, -pad, -pad}
		offsetyV = {pad, -pad, pad, -pad, pad, -pad}
		
		if DB.RuneFrame.runeOrientation == "VERTICAL" then
			self:SetWidth(3*DB.RuneFrame.widthR + 4*DB.RuneFrame.padding)
			self:SetHeight(2*DB.RuneFrame.heightR + 3*DB.RuneFrame.padding)
			for i = 1,6 do
				self:redrawStatusBar(self.Runes[i], DB.RuneFrame.widthR, DB.RuneFrame.heightR, 0, DKStatus.RuneDur or 10, point[pOrderV[i]], self, point[pOrderV[i]], offsetxV[i], offsetyV[i], DB.RuneFrame.colorR[cOrder[DB.RuneFrame.runeOrder][i]], DB.RuneFrame.colorRbg, DB.RuneFrame.runeOrientation)
			end
		else -- HORIZONTAL
			self:SetWidth(2*DB.RuneFrame.widthR + 3*DB.RuneFrame.padding)
			self:SetHeight(3*DB.RuneFrame.heightR + 4*DB.RuneFrame.padding)
			for i = 1,6 do
				self:redrawStatusBar(self.Runes[i], DB.RuneFrame.widthR, DB.RuneFrame.heightR, 0, DKStatus.RuneDur or 10, point[pOrderH[i]], self, point[pOrderH[i]], offsetxH[i], offsetyH[i], DB.RuneFrame.colorR[cOrder[DB.RuneFrame.runeOrder][i]], DB.RuneFrame.colorRbg, DB.RuneFrame.runeOrientation)
			end
		end
		for i = 1,6 do -- The labels on each individual rune
			self:redrawLabel(self.Runes[i], DB.RuneFrame.offxRlabels, DB.RuneFrame.offyRlabels, DB.RuneFrame.fontsizeRlabels, DB.RuneFrame.showRlabels, DB.RuneFrame.formatRlabels)
			self.Runes[i]:Show()
		end
		-- TODO Should I include a border option for the overall rune frame, I dont personally like them, but some probably do
		--[[BorderFrame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
											edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
											tile = true, tileSize = 16, edgeSize = 16, 
											insets = { left = 4, right = 4, top = 4, bottom = 4 }
											});]]
		--BorderFrame:SetBackdropColor(0, 0, 0, 1)
		--BorderFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1)
		-- FROST FEVER
		self:redrawStatusBar(self.FF, DB.RuneFrame.widthDots, DB.RuneFrame.heightDots, 0, aL.FrostFever.Lasts or 21, DB.RuneFrame.pointFF, self, DB.RuneFrame.pointFFparent, DB.RuneFrame.offxFF, DB.RuneFrame.offyFF, DB.RuneFrame.colorFF, DB.RuneFrame.colorFFbg, DB.RuneFrame.orientDots)
		self:redrawLabel(self.FF, DB.RuneFrame.offxDlabels, DB.RuneFrame.offyDlabels, DB.RuneFrame.fontsizeDlabels, DB.RuneFrame.showDlabels, DB.RuneFrame.formatDlabels)
		-- BLOOD PLAGUE
		self:redrawStatusBar(self.BP, DB.RuneFrame.widthDots, DB.RuneFrame.heightDots, 0, aL.BloodPlague.Lasts or 21, DB.RuneFrame.pointBP, self, DB.RuneFrame.pointBPparent, DB.RuneFrame.offxBP, DB.RuneFrame.offyBP, DB.RuneFrame.colorBP, DB.RuneFrame.colorBPbg,DB.RuneFrame.orientDots)
		self:redrawLabel(self.BP, DB.RuneFrame.offxDlabels, DB.RuneFrame.offyDlabels, DB.RuneFrame.fontsizeDlabels, DB.RuneFrame.showDlabels, DB.RuneFrame.formatDlabels)
		--RUNIC POWER
		-- The configuration screen gives user the option to autosize the RP bar to match
		-- the runeframe by setting the value to 0
		if DB.RuneFrame.widthRP == 0 then
			rpWidth = self:GetWidth()
		else
			rpWidth = DB.RuneFrame.widthRP
		end
		if DB.RuneFrame.heightRP == 0 then
			rpHeight = self:GetHeight()
		else
			rpHeight = DB.RuneFrame.heightRP
		end
		self:redrawStatusBar(self.RP, rpWidth, rpHeight, 0, DKPlayerStats.MaxRP or 100, DB.RuneFrame.pointRP, self, DB.RuneFrame.pointRPparent, DB.RuneFrame.offxRP, DB.RuneFrame.offyRP, DB.RuneFrame.colorRP, DB.RuneFrame.colorRPbg, DB.RuneFrame.orientRP)
		self:redrawLabel(self.RP, DB.RuneFrame.offxRPlabels, DB.RuneFrame.offyRPlabels, DB.RuneFrame.fontsizeRPlabels, DB.RuneFrame.showRPlabels, DB.RuneFrame.formatRPlabels)
		-- These are dividers(lines) on the RP bar showing the values of runic power dumps
		-- for example at 32, 64, 96, 128 for a glyphed frost dk
		if DB.RuneFrame.showSpark then
			-- bit of a pain, but we manage to adjust them depending on which way the RP bar is oriented
			if DB.RuneFrame.orientRP == "HORIZONTAL" then
				sparkWidth = self.RP:GetHeight()
				sparkHeight = self.RP:GetHeight() 
				sparkSpaceMultiplier = self.RP:GetWidth()/DKPlayerStats.MaxRP
				for i = 1,4 do
					self.RP.Spark[i]:SetRotation(0)
				end
			else
				sparkWidth = self.RP:GetWidth()
				sparkHeight = self.RP:GetWidth()
				sparkSpaceMultiplier = self.RP:GetHeight()/DKPlayerStats.MaxRP
				for i = 1,4 do
					self.RP.Spark[i]:SetRotation(3.14159/2) -- Rotates the line 90 degrees
				end
			end
			-- the spacing of the line depends on the spec
			if DKPlayerStats.Spec == 1 then
				sparkX[1] = (aL.RuneStrike.Cost[5]*sparkSpaceMultiplier)
			elseif DKPlayerStats.Spec == 2 then
				sparkX[1] = (aL.FrostStrike.Cost[5]*sparkSpaceMultiplier)
			else 
				sparkX[1] = (aL.DeathCoil.Cost[5]*sparkSpaceMultiplier)
			end
			sparkX[2] = 2*sparkX[1]
			sparkX[3] = 3*sparkX[1]
			sparkX[4] = 4*sparkX[1]
			for i = 1,4 do
				self.RP.Spark[i]:ClearAllPoints()
				if DB.RuneFrame.orientRP == "HORIZONTAL" then
					self.RP.Spark[i]:SetPoint("CENTER", self.RP, "LEFT", sparkX[i], 0)
				else
					self.RP.Spark[i]:SetPoint("CENTER", self.RP, "BOTTOM", 0, sparkX[i])
				end
				self.RP.Spark[i]:SetHeight(sparkHeight)
				self.RP.Spark[i]:SetWidth(sparkWidth)
				-- whether to draw 3 or 4 depends on the max RP and the runic power cost
				if (DB.RuneFrame.orientRP == "HORIZONTAL" and sparkX[i] < self.RP:GetWidth()) or (DB.RuneFrame.orientRP == "VERTICAL" and sparkX[i] < self.RP:GetHeight()) then
					self.RP.Spark[i]:Show()
				else
					self.RP.Spark[i]:Hide()
				end
			end
		else
			for i = 1,4 do
				self.RP.Spark[i]:Hide()
			end
		end
		
		if DB.RuneFrame.showDots then
			self.FF:Show()
			self.BP:Show()
		else
			self.FF:Hide()
			self.BP:Hide()
		end
		if DB.RuneFrame.showRP then
			self.RP:Show()
		else
			self.RP:Hide()
		end
		-- Buttons
		-- They are spec specific
		if DKPlayerStats.Spec == 1 then
			textureB1 = aL.ScentOfBlood.Icon
			self.B1.label:SetText("1")
			textureB2 = aL.RuneTap.Icon
			self.B2.label:SetText("")
			textureB3 = aL.RaiseDead.Icon
			textureB4 = aL.BoneShield.Icon
		elseif DKPlayerStats.Spec == 2 then
			textureB1 = aL.Rime.Icon
			self.B1.label:SetText("2")
			textureB2 = aL.KillingMachine.Icon
			self.B2.label:SetText("")
			textureB3 = aL.RaiseDead.Icon
			textureB4 = aL.RuneOfTheFallenCrusader.Icon
		else
			textureB1 = aL.ShadowInfusion.Icon
			textureB2 = aL.SuddenDoom.Icon
			self.B2.label:SetText("2")
			textureB3 = aL.UnholyFrenzy.Icon
			textureB4 = aL.RuneOfTheFallenCrusader.Icon
		end
		-- Button 1
		self:redrawButton(self.B1, DB.RuneFrame.sizeB1, textureB1, DB.RuneFrame.pointB1, self, DB.RuneFrame.pointB1parent, DB.RuneFrame.offxB1, DB.RuneFrame.offyB1)
		self:redrawButtonLabel(self.B1, DB.RuneFrame.offxB1labels, DB.RuneFrame.offyB1labels, DB.RuneFrame.fontsizeB1labels, DB.RuneFrame.fontColorB1, DB.RuneFrame.showB1labels)
		if DB.RuneFrame.showB1 then
			self.B1:Show()
		else
			self.B1:Hide()
		end
		-- Button 2
		self:redrawButton(self.B2, DB.RuneFrame.sizeB2, textureB2, DB.RuneFrame.pointB2, self, DB.RuneFrame.pointB2parent, DB.RuneFrame.offxB2, DB.RuneFrame.offyB2)
		self:redrawButtonLabel(self.B2, DB.RuneFrame.offxB2labels, DB.RuneFrame.offyB2labels, DB.RuneFrame.fontsizeB2labels, DB.RuneFrame.fontColorB2, DB.RuneFrame.showB2labels)
		if DB.RuneFrame.showB2 then
			self.B2:Show()
		else
			self.B2:Hide()
		end
		-- Button 3
		self:redrawButton(self.B3, DB.RuneFrame.sizeB3, textureB3, DB.RuneFrame.pointB3, self, DB.RuneFrame.pointB3parent, DB.RuneFrame.offxB3, DB.RuneFrame.offyB3)
		self:redrawButtonLabel(self.B3, DB.RuneFrame.offxB3labels, DB.RuneFrame.offyB3labels, DB.RuneFrame.fontsizeB3labels, DB.RuneFrame.fontColorB3, DB.RuneFrame.showB3labels)
		if DB.RuneFrame.showB3 then
			self.B3:Show()
		else
			self.B3:Hide()
		end
		-- Button 4
		self:redrawButton(self.B4, DB.RuneFrame.sizeB4, textureB4, DB.RuneFrame.pointB4, self, DB.RuneFrame.pointB4parent, DB.RuneFrame.offxB4, DB.RuneFrame.offyB4)
		self:redrawButtonLabel(self.B4, DB.RuneFrame.offxB4labels, DB.RuneFrame.offyB4labels, DB.RuneFrame.fontsizeB4labels, DB.RuneFrame.fontColorB4, DB.RuneFrame.showB4labels)
		if DB.RuneFrame.showB4 then
			self.B4:Show()
		else
			self.B4:Hide()
		end
		-- Blood Shield
		-- like the RP bar, the BS bar autosizes when the values are set to 0
		if DB.RuneFrame.widthBS == 0 then
			bsWidth = self:GetWidth()
		else
			bsWidth = DB.RuneFrame.widthBS
		end
		if DB.RuneFrame.heightBS == 0 then
			bsHeight = self:GetHeight()
		else
			bsHeight = DB.RuneFrame.heightBS
		end
		self:redrawStatusBar(self.BS, bsWidth, bsHeight, 0, DKStatus.MaxHealth, DB.RuneFrame.pointBS, self, DB.RuneFrame.pointBSparent, DB.RuneFrame.offxBS, DB.RuneFrame.offyBS, DB.RuneFrame.colorBS, DB.RuneFrame.colorBSbg, DB.RuneFrame.orientBS)
		self:redrawLabel(self.BS, DB.RuneFrame.offxBSlabels, DB.RuneFrame.offyBSlabels, DB.RuneFrame.fontsizeBSlabels, DB.RuneFrame.showBSlabels, DB.RuneFrame.formatBSlabels)
		-- Blood Shields last 10 secs, so we have a small spark that moves across the bar
		-- to indicate how much time is left
		self.BS.Spark:SetWidth(DB.RuneFrame.sparkWidth)
		self.BS.Spark:SetHeight(DB.RuneFrame.sparkHeight)
		BSsparkXMultiplier = self.BS:GetWidth()/10 -- converts the duration to pixels
		
		if DB.RuneFrame.show then
			self:Show()
		else
			self:Hide()
		end
	end
	
	local function updateRuneFrame(self) -- This function is getting called on every OnUpdate when the runeframe is enabled
		for i = 1,6 do
			-- This bit of voodoo is taking the values from the rune CDs in BUF order
			-- and transforming them into the user specified order
			runeValues[rOrder[DB.RuneFrame.runeOrder][i]] = DKStatus[runeorder[i]]
			notDeathRunes[rOrder[DB.RuneFrame.runeOrder][i]] = DKStatus.NotDeath[i]
			-- we check whether the rune should be flashed
			if DB.RuneFrame.doFlash then
				if runeValues[i] <= 0 then
					if runeFlash[i] == 0 then
						runeFlash[i] = DKStatus.Time
					end
					if DKStatus.Time - runeFlash[i] < DB.RuneFrame.flashTimer then
						flashTrue[i] = true
					else
						flashTrue[i] = false
					end
				else
					flashTrue[i] = false
					runeFlash[i] = 0
				end
			else
				flashTrue[i] = false
				runeFlash[i] = 0
			end
		end
		for i = 1,3 do
			-- For each pair of rune types, we order them with the one that will finish its CD first
			if runeValues[rune1order[i]] > runeValues[rune2order[i]] then
				runeValues[rune1order[i]], runeValues[rune2order[i]] = runeValues[rune2order[i]], runeValues[rune1order[i]]
				flashTrue[rune1order[i]], flashTrue[rune2order[i]] = flashTrue[rune2order[i]], flashTrue[rune1order[i]]
				notDeathRunes[rune1order[i]], notDeathRunes[rune2order[i]] = notDeathRunes[rune2order[i]], notDeathRunes[rune1order[i]]
			end
		end
		for i = 1,6 do
			
			-- Here we actually set the color for each rune
			if runeValues[i] == 0 then
				if  flashTrue[i] then 
					if not notDeathRunes[i] then
						self.Runes[i]:SetStatusBarColor(unpack(DB.RuneFrame.colorRlight[4]))
					else
						self.Runes[i]:SetStatusBarColor(unpack(DB.RuneFrame.colorRlight[cOrder[DB.RuneFrame.runeOrder][i]]))
					end
				else
					if not notDeathRunes[i] then
						self.Runes[i]:SetStatusBarColor(unpack(DB.RuneFrame.colorR[4]))
					else
						self.Runes[i]:SetStatusBarColor(unpack(DB.RuneFrame.colorR[cOrder[DB.RuneFrame.runeOrder][i]]))
					end
				end
			else
				if not notDeathRunes[i] then
					self.Runes[i]:SetStatusBarColor(unpack(DB.RuneFrame.colorRdark[4]))
				else
					self.Runes[i]:SetStatusBarColor(unpack(DB.RuneFrame.colorRdark[cOrder[DB.RuneFrame.runeOrder][i]]))
				end
			end
		end
		for i = 1,6 do
			self:setRuneLabelSuffix(self.Runes[i], DKStatus.RuneDur)
			self.Runes[i]:SetMinMaxValues(0,DKStatus.RuneDur)
			-- Here we set the value for each rune to display, a 0 cd is a full bar
			self.Runes[i]:SetValue(DKStatus.RuneDur - runeValues[i])
			
		end
		if DB.RuneFrame.showRlabels then
			if DB.RuneFrame.formatRlabels == 2 or DB.RuneFrame.formatRlabels == 4 then
				for i =1,6 do
					self.Runes[i].labelprefix = floor(runeValues[i] + .5)
				end
			else
				for i =1,6 do
					self.Runes[i].labelprefix = format("%.1f", runeValues[i])
				end
			end
			for i =1,6 do
				self.Runes[i].label:SetText(self.Runes[i].labelprefix..self.Runes[i].labelsuffix)
			end
		end
		-- Update the dot timers
		if DB.RuneFrame.showDots then
			self.FF:SetValue(DKStatus.FF)
			self.BP:SetValue(DKStatus.BP)
			if DB.RuneFrame.formatDlabels == 2 or DB.RuneFrame.formatDlabels == 4 then
				self.FF.labelprefix = floor(DKStatus.FF + .5)
				self.BP.labelprefix = floor(DKStatus.BP + .5)
			else
				self.FF.labelprefix = format("%.1f", DKStatus.FF)
				self.BP.labelprefix = format("%.1f", DKStatus.BP)
			end
			self.FF.label:SetText(self.FF.labelprefix..self.FF.labelsuffix)
			self.BP.label:SetText(self.BP.labelprefix..self.BP.labelsuffix)
		end
		-- Update the runic power bar
		if DB.RuneFrame.showRP then
			self.RP:SetValue(DKStatus.RP)
			self.RP.labelprefix = DKStatus.RP
			self.RP.label:SetText(self.RP.labelprefix..self.RP.labelsuffix)
		end
		-- Update the Buttons
		if DKPlayerStats.Spec == 1 then
			-- Button 1 Scent of Blood
			if DKStatus.SoB > 0 and DB.RuneFrame.showB1 then
				self.B1.label:SetText(DKStatus.SoB)
				self.B1:Show()
				self.B1.label:Show()
			else
				self.B1:Hide()
				self.B1.label:Hide()
			end
			-- Button 2 Will of Necropolis
			if DKStatus.WON and DB.RuneFrame.showB2 then
				self.B2:Show()
			else
				self.B2:Hide()
			end
			-- Button 3 Ghoul
			if DKStatus.PetDur > 0 and DB.RuneFrame.showB3 then
				self.B3.label:SetFormattedText("%d",DKStatus.PetDur)
				self.B3:Show()
			else
				self.B3:Hide()
			end
			-- Button 4 Bone Shield
			if DKStatus.Bone > 0 and DB.RuneFrame.showB4 then
				self.B4.label:SetText(DKStatus.Bone)
				self.B4:Show()
			else
				self.B4:Hide()
			end
		elseif DKPlayerStats.Spec == 2 then
			-- Button 1 Rime
			if DKStatus.Rime > 0 and DB.RuneFrame.showB1 then
				if DKStatus.Rime > 1 then
					self.B1.label:SetText("2")
					self.B1.label:Show()
				else
					self.B1.label:SetText("")
					self.B1.label:Hide()
				end
				self.B1:Show()
			else
				self.B1:Hide()
				self.B1.label:SetText("")
				self.B1.label:Hide()
			end
			-- Button 2 Killing Machine
			if DKStatus.KM and DB.RuneFrame.showB2 then
				self.B2:Show()
			else
				self.B2:Hide()
			end
			-- Button 3 Pillar of Frost/BloodLust/Ghoul/PillarCD
			if DKStatus.Pillar > 0 and DB.RuneFrame.showB3 then
				self.B3.texture:SetTexture(aL.PillarOfFrost.Icon)
				self.B3.label:SetFormattedText("%d",DKStatus.Pillar)
				self.B3:Show()
			elseif DKStatus.BL > 0 and DB.RuneFrame.showB3 then
				self.B3.texture:SetTexture(aL.Bloodlust.Icon)
				self.B3.label:SetFormattedText("%d",DKStatus.BL)
				self.B3:Show()
			elseif DKStatus.PetDur > 0 and DB.RuneFrame.showB3 then
				self.B3.texture:SetTexture(aL.RaiseDead.Icon)
				self.B3.label:SetFormattedText("%d",DKStatus.PetDur)
				self.B3:Show()
			--elseif DKStatus.PillarCD > 0 and DB.RuneFrame.showB3 then
				--self.B3.texture:SetTexture(aL.PillarOfFrost.Icon)
				--self.B3.label:SetText(format("%d",DKStatus.PillarCD))
				--self.B3:Show()
			else
				self.B3:Hide()
			end
			-- Button 4 Rune of the Fallen CRusader
			if DKStatus.RotFC > 0 and DB.RuneFrame.showB4 then
				self.B4.label:SetFormattedText("%d",DKStatus.RotFC)
				self.B4:Show()
			else
				self.B4:Hide()
			end
		else -- For unholy the texture on button 1 changes depending whether pet is hulked out
			-- Button 1 Hulk/Shadow Infusion
			if DKStatus.Hulk > 0  and DB.RuneFrame.showB1 then
				self.B1.texture:SetTexture(aL.DarkTransformation.Icon)
				self.B1.label:SetFormattedText("%d",DKStatus.Hulk)
				self.B1.label:Show()
				self.B1:Show()
			elseif DKStatus.SI > 0 and DB.RuneFrame.showB1 then
				self.B1.texture:SetTexture(aL.ShadowInfusion.Icon)
				self.B1.label:SetText(DKStatus.SI)
				self.B1.label:Show()
				self.B1:Show()
			else
				self.B1.label:Hide()
				self.B1:Hide()
			end
			-- Button 2 Sudden Doom
			if DKStatus.Doom > 0 and DB.RuneFrame.showB2 then
				if DKStatus.Doom > 1 then
					self.B2.label:Show()
				else
					self.B2.label:Hide()
				end
				self.B2:Show()
			else
				self.B2:Hide()
				self.B2.label:Hide()
			end
			-- Button 3 Unholy Frenzy
			if DKStatus.BL > 0 and DB.RuneFrame.showB3 then
				self.B3.texture:SetTexture(aL.Bloodlust.Icon)
				self.B3.label:SetFormattedText("%d",DKStatus.BL)
				self.B3:Show()
			elseif DKStatus.UF > 0 and DB.RuneFrame.showB3 then
				self.B3.texture:SetTexture(aL.UnholyFrenzy.Icon)
				self.B3.label:SetFormattedText("%d",DKStatus.UF)
				self.B3:Show()
			else
				self.B3:Hide()
			end
			-- Button 4 Rune of the Fallen Crusader
			if DKStatus.RotFC > 0 and DB.RuneFrame.showB4 then
				self.B4.label:SetFormattedText("%d",DKStatus.RotFC)
				self.B4:Show()
			else
				self.B4:Hide()
			end
		end
		-- Update the blood shield bar
		if DKPlayerStats.Spec == 1 and DB.RuneFrame.showBS then
			if DKStatus.BS == 0 and not DB.RuneFrame.showBSempty then
				self.BS:Hide()
			else
				self.BS:SetValue(DKStatus.BS)
				self.BS:SetMinMaxValues(0,DKStatus.MaxHealth)
				if not DB.RuneFrame.showBSlabels or (DKStatus.BS == 0 and not DB.RuneFrame.showBSlabelszero) then
					self.BS.label:Hide()
				else
					self.BS.labelprefix = floor(DKStatus.BS/1000 + .5).."k"
					self.BS.label:SetText(self.BS.labelprefix)
					self.BS.label:Show()
				end
				if DKStatus.BSDur > 0 and DB.RuneFrame.showBSSpark then
					self.BS.Spark:ClearAllPoints()
					self.BS.Spark:SetPoint("BOTTOM", self.BS, "BOTTOMLEFT", DKStatus.BSDur*BSsparkXMultiplier, 0)
					self.BS.Spark:Show()
				else
					self.BS.Spark:Hide()
				end
				self.BS:Show()
			end
		else
			self.BS:Hide()
		end
	end
	
	DB = DKOneDB -- local version of the database
	-- Create all the components of the Runeframe
	local RuneFrame = CreateFrame("Frame", "outerframe" , parent )
	RuneFrame.bg = RuneFrame:CreateTexture()
	RuneFrame.bg:SetAllPoints(RuneFrame)
	RuneFrame.bg:SetTexture(unpack(DB.RuneFrame.colorRFbg))
	RuneFrame.Runes = {}
	for i = 1,6 do
		RuneFrame.Runes[i] = makeStatusBar(tostring("R"..i), RuneFrame)
		RuneFrame.Runes[i]:Show()
	end
	RuneFrame.FF = makeStatusBar("FF", RuneFrame)
	RuneFrame.BP = makeStatusBar("BP", RuneFrame)
	RuneFrame.RP = makeStatusBar("RP", RuneFrame)
	RuneFrame.RP.Spark = {}
	for i = 1,4 do 
		RuneFrame.RP.Spark[i] = RuneFrame.RP:CreateTexture(nil, "OVERLAY")
		--RuneFrame.RP.Spark[i]:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		RuneFrame.RP.Spark[i]:SetTexture(sparkTex)
		--self.RP.Spark1:SetVertexColor(DB.RuneFrame.colorRPspark)
		RuneFrame.RP.Spark[i]:SetBlendMode("ADD")
		RuneFrame.RP.Spark[i]:SetVertTile(false)
		RuneFrame.RP.Spark[i]:SetHorizTile(false)
		RuneFrame.RP.Spark[i]:SetWidth(8)
	end
	
	RuneFrame.B1 = makeButton("B1", RuneFrame)
	RuneFrame.B2 = makeButton("B2", RuneFrame)
	RuneFrame.B3 = makeButton("B3", RuneFrame)
	RuneFrame.B4 = makeButton("B4", RuneFrame)
	RuneFrame.BS = makeStatusBar("BS", RuneFrame)
	RuneFrame.BS.Spark = RuneFrame.BS:CreateTexture(nil, "OVERLAY")
	RuneFrame.BS.Spark:SetTexture(sparkTex)
	--self.RP.Spark1:SetVertexColor(DB.RuneFrame.colorRPspark) -- Cant get this to work right
	RuneFrame.BS.Spark:SetBlendMode("ADD")
	
	-- set local functions as methods
	RuneFrame.redrawStatusBar = redrawStatusBar
	RuneFrame.redrawLabel = redrawLabel
	RuneFrame.redrawRuneFrame = redrawRuneFrame
	RuneFrame.updateRuneFrame = updateRuneFrame
	RuneFrame.redrawButton = redrawButton
	RuneFrame.redrawButtonLabel = redrawButtonLabel
	RuneFrame.updateRunDur = updateRunDur
	RuneFrame.setRuneLabelSuffix = setRuneLabelSuffix
	RuneFrame.redrawRuneSpark = redrawRuneSpark
	RuneFrame:redrawRuneFrame()
	
	return RuneFrame
end	
